<?php
/**
 * The Header: Logo and main menu
 *
 * @package FRIBBO
 * @since FRIBBO 1.0
 */
?><!DOCTYPE html>
<html <?php language_attributes(); ?> class="no-js
									<?php
										// Class scheme_xxx need in the <html> as context for the <body>!
										echo ' scheme_' . esc_attr( fribbo_get_theme_option( 'color_scheme' ) );
									?>
										">
<head>
	<?php wp_head(); ?>
</head>

<body <?php body_class(); ?>>

	<?php
	if ( function_exists( 'wp_body_open' ) ) {
		wp_body_open();
	} else {
		do_action( 'wp_body_open' );
	}
	do_action( 'fribbo_action_before_body' );
	?>

	<div class="body_wrap">

		<div class="page_wrap">
			
			<?php
			$fribbo_full_post_loading = ( is_singular( 'post' ) || is_singular( 'attachment' ) ) && fribbo_get_value_gp( 'action' ) == 'full_post_loading';
			$fribbo_prev_post_loading = ( is_singular( 'post' ) || is_singular( 'attachment' ) ) && fribbo_get_value_gp( 'action' ) == 'prev_post_loading';

			// Don't display the header elements while actions 'full_post_loading' and 'prev_post_loading'
			if ( ! $fribbo_full_post_loading && ! $fribbo_prev_post_loading ) {

				// Short links to fast access to the content, sidebar and footer from the keyboard
				?>
				<a class="fribbo_skip_link skip_to_content_link" href="#content_skip_link_anchor" tabindex="1"><?php esc_html_e( "Skip to content", 'fribbo' ); ?></a>
				<?php if ( fribbo_sidebar_present() ) { ?>
				<a class="fribbo_skip_link skip_to_sidebar_link" href="#sidebar_skip_link_anchor" tabindex="1"><?php esc_html_e( "Skip to sidebar", 'fribbo' ); ?></a>
				<?php } ?>
				<a class="fribbo_skip_link skip_to_footer_link" href="#footer_skip_link_anchor" tabindex="1"><?php esc_html_e( "Skip to footer", 'fribbo' ); ?></a>
				
				<?php
				do_action( 'fribbo_action_before_header' );

				// Header
				$fribbo_header_type = fribbo_get_theme_option( 'header_type' );
				if ( 'custom' == $fribbo_header_type && ! fribbo_is_layouts_available() ) {
					$fribbo_header_type = 'default';
				}
				get_template_part( apply_filters( 'fribbo_filter_get_template_part', "templates/header-{$fribbo_header_type}" ) );

				// Side menu
				if ( in_array( fribbo_get_theme_option( 'menu_side' ), array( 'left', 'right' ) ) ) {
					get_template_part( apply_filters( 'fribbo_filter_get_template_part', 'templates/header-navi-side' ) );
				}

				// Mobile menu
				get_template_part( apply_filters( 'fribbo_filter_get_template_part', 'templates/header-navi-mobile' ) );

				do_action( 'fribbo_action_after_header' );

			}
			?>

			<div class="page_content_wrap">
				<?php
				do_action( 'fribbo_action_page_content_wrap', $fribbo_full_post_loading || $fribbo_prev_post_loading );

				// Single posts banner
				if ( is_singular( 'post' ) || is_singular( 'attachment' ) ) {
					if ( $fribbo_prev_post_loading ) {
						if ( fribbo_get_theme_option( 'posts_navigation_scroll_which_block' ) != 'article' ) {
							do_action( 'fribbo_action_between_posts' );
						}
					} else {
					}
					// Single post thumbnail and title
					get_template_part( apply_filters( 'fribbo_filter_get_template_part', 'templates/single-styles/' . fribbo_get_theme_option( 'single_style' ) ) );
				}

				// Widgets area above page content
				$fribbo_body_style   = fribbo_get_theme_option( 'body_style' );
				$fribbo_widgets_name = fribbo_get_theme_option( 'widgets_above_page' );
				$fribbo_show_widgets = ! fribbo_is_off( $fribbo_widgets_name ) && is_active_sidebar( $fribbo_widgets_name );
				if ( $fribbo_show_widgets ) {
					if ( 'fullscreen' != $fribbo_body_style ) {
						?>
						<div class="content_wrap">
							<?php
					}
					fribbo_create_widgets_area( 'widgets_above_page' );
					if ( 'fullscreen' != $fribbo_body_style ) {
						?>
						</div><!-- </.content_wrap> -->
						<?php
					}
				}
				// Content area
				?>
				<div class="content_wrap<?php echo 'fullscreen' == $fribbo_body_style ? '_fullscreen' : ''; ?>">
						<?php
						// Share FIX
						if ( is_single() && fribbo_is_on( fribbo_get_theme_option( 'show_share_links' ) ) && fribbo_is_on( fribbo_get_theme_option( 'show_share_links_fix' ) )) {
							fribbo_show_share_links(
								array(
									'type'    => 'block',
									'caption' => '',
									'before'  => '<span class="side_share post_share">',
									'after'   => '</span>',
								)
							);
						}
						?>
					<div class="content<?php
						if ( is_singular( 'post' ) || is_singular( 'attachment' ) ) {
							echo ' content_single';
						}
					?>">
						<?php
						// Skip link anchor to fast access to the content from keyboard
						?>
						<a id="content_skip_link_anchor" class="fribbo_skip_link_anchor" href="#"></a>
						<?php
						// Single posts banner between prev/next posts
						if ( ( is_singular( 'post' ) || is_singular( 'attachment' ) )
							&& $fribbo_prev_post_loading 
							&& fribbo_get_theme_option( 'posts_navigation_scroll_which_block' ) == 'article'
						) {
							do_action( 'fribbo_action_between_posts' );
						}
						// Additional wrap of the content of the single post or attachment
						if ( is_singular( 'post' ) || is_singular( 'attachment' ) ) {
							?>
							<div class="content_inner">
							<?php
						}

						// Widgets area inside page content
						fribbo_create_widgets_area( 'widgets_above_content' );
