<?php
/**
 * The main template file.
 *
 * This is the most generic template file in a WordPress theme
 * and one of the two required files for a theme (the other being style.css).
 * It is used to display a page when nothing more specific matches a query.
 * E.g., it puts together the home page when no home.php file exists.
 * Learn more: //codex.wordpress.org/Template_Hierarchy
 *
 * @package FRIBBO
 * @since FRIBBO 1.0
 */

$fribbo_template = apply_filters( 'fribbo_filter_get_template_part', fribbo_blog_archive_get_template() );

if ( ! empty( $fribbo_template ) && 'index' != $fribbo_template ) {

	get_template_part( $fribbo_template );

} else {

	fribbo_storage_set( 'blog_archive', true );

	get_header();

	if ( have_posts() ) {

		// Query params
		$fribbo_stickies  = is_home() ? get_option( 'sticky_posts' ) : false;
		$fribbo_post_type = fribbo_get_theme_option( 'post_type' );
		$fribbo_args      = array(
								'blog_style'     => fribbo_get_theme_option( 'blog_style' ),
								'post_type'      => $fribbo_post_type,
								'taxonomy'       => fribbo_get_post_type_taxonomy( $fribbo_post_type ),
								'parent_cat'     => fribbo_get_theme_option( 'parent_cat' ),
								'posts_per_page' => fribbo_get_theme_option( 'posts_per_page' ),
								'sticky'         => fribbo_get_theme_option( 'sticky_style' ) == 'columns'
															&& is_array( $fribbo_stickies )
															&& count( $fribbo_stickies ) > 0
															&& get_query_var( 'paged' ) < 1
								);

		fribbo_blog_archive_start();

		do_action( 'fribbo_action_blog_archive_start' );

		if ( is_author() ) {
			do_action( 'fribbo_action_before_page_author' );
			get_template_part( apply_filters( 'fribbo_filter_get_template_part', 'templates/author-page' ) );
			do_action( 'fribbo_action_after_page_author' );
		}

		if ( fribbo_get_theme_option( 'show_filters' ) ) {
			do_action( 'fribbo_action_before_page_filters' );
			fribbo_show_filters( $fribbo_args );
			do_action( 'fribbo_action_after_page_filters' );
		} else {
			do_action( 'fribbo_action_before_page_posts' );
			fribbo_show_posts( array_merge( $fribbo_args, array( 'cat' => $fribbo_args['parent_cat'] ) ) );
			do_action( 'fribbo_action_after_page_posts' );
		}

		do_action( 'fribbo_action_blog_archive_end' );

		fribbo_blog_archive_end();

	} else {

		if ( is_search() ) {
			get_template_part( apply_filters( 'fribbo_filter_get_template_part', 'templates/content', 'none-search' ), 'none-search' );
		} else {
			get_template_part( apply_filters( 'fribbo_filter_get_template_part', 'templates/content', 'none-archive' ), 'none-archive' );
		}
	}

	get_footer();
}
