<?php
/**
 * The template to display single post
 *
 * @package FRIBBO
 * @since FRIBBO 1.0
 */

// Full post loading
$full_post_loading        = fribbo_get_value_gp( 'action' ) == 'full_post_loading';

// Prev post loading
$prev_post_loading        = fribbo_get_value_gp( 'action' ) == 'prev_post_loading';
$prev_post_loading_type   = fribbo_get_theme_option( 'posts_navigation_scroll_which_block' );

// Position of the related posts
$fribbo_related_position = fribbo_get_theme_option( 'related_position' );

// Type of the prev/next posts navigation
$fribbo_posts_navigation = fribbo_get_theme_option( 'posts_navigation' );
$fribbo_prev_post        = false;

// Rewrite style of the single post if current post loading via AJAX and featured image and title is not in the content
if ( ( $full_post_loading 
		|| 
		( $prev_post_loading && 'article' == $prev_post_loading_type )
	) 
	&& 
	! in_array( fribbo_get_theme_option( 'single_style' ), array( 'in-above', 'in-below', 'in-over', 'in-sticky' ) )
) {
	fribbo_storage_set_array( 'options_meta', 'single_style', 'in-below' );
}

get_header();

while ( have_posts() ) {
	the_post();

	// Type of the prev/next posts navigation
	if ( 'scroll' == $fribbo_posts_navigation ) {
		$fribbo_prev_post = get_previous_post( true );         // Get post from same category
		if ( ! $fribbo_prev_post ) {
			$fribbo_prev_post = get_previous_post( false );    // Get post from any category
			if ( ! $fribbo_prev_post ) {
				$fribbo_posts_navigation = 'links';
			}
		}
	}

	// Override some theme options to display featured image, title and post meta in the dynamic loaded posts
	if ( $full_post_loading || ( $prev_post_loading && $fribbo_prev_post ) ) {
		fribbo_sc_layouts_showed( 'featured', false );
		fribbo_sc_layouts_showed( 'title', false );
		fribbo_sc_layouts_showed( 'postmeta', false );
	}

	// If related posts should be inside the content
	if ( strpos( $fribbo_related_position, 'inside' ) === 0 ) {
		ob_start();
	}

	// Display post's content
	get_template_part( apply_filters( 'fribbo_filter_get_template_part', 'templates/content', 'single-' . fribbo_get_theme_option( 'single_style' ) ), 'single-' . fribbo_get_theme_option( 'single_style' ) );

	// If related posts should be inside the content
	if ( strpos( $fribbo_related_position, 'inside' ) === 0 ) {
		$fribbo_content = ob_get_contents();
		ob_end_clean();

		ob_start();
		do_action( 'fribbo_action_related_posts' );
		$fribbo_related_content = ob_get_contents();
		ob_end_clean();

		$fribbo_related_position_inside = max( 0, min( 9, fribbo_get_theme_option( 'related_position_inside' ) ) );
		if ( 0 == $fribbo_related_position_inside ) {
			$fribbo_related_position_inside = mt_rand( 1, 9 );
		}

		$fribbo_p_number = 0;
		$fribbo_related_inserted = false;
		for ( $i = 0; $i < strlen( $fribbo_content ) - 3; $i++ ) {
			if ( '<' == $fribbo_content[ $i ] && 'p' == $fribbo_content[ $i + 1 ] && in_array( $fribbo_content[ $i + 2 ], array( '>', ' ' ) ) ) {
				$fribbo_p_number++;
				if ( $fribbo_related_position_inside == $fribbo_p_number ) {
					$fribbo_related_inserted = true;
					$fribbo_content = ( $i > 0 ? substr( $fribbo_content, 0, $i ) : '' )
										. $fribbo_related_content
										. substr( $fribbo_content, $i );
				}
			}
		}
		if ( ! $fribbo_related_inserted ) {
			$fribbo_content .= $fribbo_related_content;
		}

		fribbo_show_layout( $fribbo_content );
	}

	// Author bio
	if ( fribbo_get_theme_option( 'show_author_info' ) == 1
		&& ! is_attachment()
		&& get_the_author_meta( 'description' )
		&& ( 'scroll' != $fribbo_posts_navigation || fribbo_get_theme_option( 'posts_navigation_scroll_hide_author' ) == 0 )
		&& ( ! $full_post_loading || fribbo_get_theme_option( 'open_full_post_hide_author' ) == 0 )
	) {
		do_action( 'fribbo_action_before_post_author' );
		get_template_part( apply_filters( 'fribbo_filter_get_template_part', 'templates/author-bio' ) );
		do_action( 'fribbo_action_after_post_author' );
	}

	// Previous/next post navigation.
	if ( 'links' == $fribbo_posts_navigation && ! $full_post_loading ) {
		do_action( 'fribbo_action_before_post_navigation' );
		?>
		<div class="nav-links-single<?php
			if ( ! fribbo_is_off( fribbo_get_theme_option( 'posts_navigation_fixed' ) ) ) {
				echo ' nav-links-fixed fixed';
			}
		?>">
			<?php
			the_post_navigation(
				array(
					'next_text' => '<span class="nav-arrow"></span>'
						. '<span class="screen-reader-text">' . esc_html__( 'Next', 'fribbo' ) . '</span> '
						. '<h6 class="post-title">%title</h6>',
					'prev_text' => '<span class="nav-arrow"></span>'
						. '<span class="screen-reader-text">' . esc_html__( 'Prev', 'fribbo' ) . '</span> '
						. '<h6 class="post-title">%title</h6>',
				)
			);
			?>
		</div>
		<?php
		do_action( 'fribbo_action_after_post_navigation' );
	}

	// Related posts
	if ( 'below_content' == $fribbo_related_position
		&& ( 'scroll' != $fribbo_posts_navigation || fribbo_get_theme_option( 'posts_navigation_scroll_hide_related' ) == 0 )
		&& ( ! $full_post_loading || fribbo_get_theme_option( 'open_full_post_hide_related' ) == 0 )
	) {
		do_action( 'fribbo_action_related_posts' );
	}

	// If comments are open or we have at least one comment, load up the comment template.
	$fribbo_comments_number = get_comments_number();
	if ( comments_open() || $fribbo_comments_number > 0 ) {
		if ( fribbo_get_value_gp( 'show_comments' ) == 1 || ( ! $full_post_loading && ( 'scroll' != $fribbo_posts_navigation || fribbo_get_theme_option( 'posts_navigation_scroll_hide_comments' ) == 0 || fribbo_check_url( '#comment' ) ) ) ) {
			do_action( 'fribbo_action_before_comments' );
			comments_template();
			do_action( 'fribbo_action_after_comments' );
		} else {
			?>
			<div class="show_comments_single">
				<a href="<?php echo esc_url( add_query_arg( array( 'show_comments' => 1 ), get_comments_link() ) ); ?>" class="theme_button show_comments_button">
					<?php
					if ( $fribbo_comments_number > 0 ) {
						echo esc_html( sprintf( _n( 'Show comment', 'Show comments ( %d )', $fribbo_comments_number, 'fribbo' ), $fribbo_comments_number ) );
					} else {
						esc_html_e( 'Leave a comment', 'fribbo' );
					}
					?>
				</a>
			</div>
			<?php
		}
	}

	if ( 'scroll' == $fribbo_posts_navigation && ! $full_post_loading ) {
		?>
		<div class="nav-links-single-scroll"
			data-post-id="<?php echo esc_attr( get_the_ID( $fribbo_prev_post ) ); ?>"
			data-post-link="<?php echo esc_attr( get_permalink( $fribbo_prev_post ) ); ?>"
			data-post-title="<?php the_title_attribute( array( 'post' => $fribbo_prev_post ) ); ?>">
		</div>
		<?php
	}
}

get_footer();
