<?php
/* Download Monitor support functions
------------------------------------------------------------------------------- */

// Theme init priorities:
// 9 - register other filters (for installer, etc.)
if ( ! function_exists( 'fribbo_download_monitor_theme_setup9' ) ) {
	add_action( 'after_setup_theme', 'fribbo_download_monitor_theme_setup9', 9 );
	function fribbo_download_monitor_theme_setup9() {
		if ( fribbo_exists_download_monitor() ) {
			add_action( 'wp_enqueue_scripts', 'fribbo_download_monitor_frontend_scripts', 1100 );
			add_filter( 'fribbo_filter_merge_styles', 'fribbo_download_monitor_merge_styles' );
		}
		if ( is_admin() ) {
			add_filter( 'fribbo_filter_tgmpa_required_plugins', 'fribbo_download_monitor_tgmpa_required_plugins' );
		}
	}
}

// Filter to add in the required plugins list
if ( ! function_exists( 'fribbo_download_monitor_tgmpa_required_plugins' ) ) {
	//Handler of the add_filter('fribbo_filter_tgmpa_required_plugins',	'fribbo_download_monitor_tgmpa_required_plugins');
	function fribbo_download_monitor_tgmpa_required_plugins( $list = array() ) {
		if ( fribbo_storage_isset( 'required_plugins', 'download-monitor' ) && fribbo_storage_get_array( 'required_plugins', 'download-monitor', 'install' ) !== false ) {
			$list[] = array(
				'name'     => fribbo_storage_get_array( 'required_plugins', 'download-monitor', 'title' ),
				'slug'     => 'download-monitor',
				'required' => false,
			);
		}
		return $list;
	}
}

// Check if Instagram Feed installed and activated
if ( ! function_exists( 'fribbo_exists_download_monitor' ) ) {
	function fribbo_exists_download_monitor() {
		return defined( 'DLM_VERSION' );
	}
}

// Enqueue styles for frontend
if ( ! function_exists( 'fribbo_download_monitor_frontend_scripts' ) ) {
	//Handler of the add_action( 'wp_enqueue_scripts', 'fribbo_download_monitor_frontend_scripts', 1100 );
	function fribbo_download_monitor_frontend_scripts() {
		if ( fribbo_is_on( fribbo_get_theme_option( 'debug_mode' ) ) ) {
			$fribbo_url = fribbo_get_file_url( 'plugins/download-monitor/download-monitor.css' );

			if ( '' != $fribbo_url ) {
				wp_enqueue_style( 'fribbo-download-monitor', $fribbo_url, array(), null );
			}

		}
	}
}

// Merge custom styles
if ( ! function_exists( 'fribbo_download_monitor_merge_styles' ) ) {
	//Handler of the add_filter('fribbo_filter_merge_styles', 'fribbo_download_monitor_merge_styles');
	function fribbo_download_monitor_merge_styles( $list ) {
		$list[] = 'plugins/download-monitor/download-monitor.css';
		return $list;
	}
}



// add meta count
if(fribbo_exists_download_monitor()){

	function fribbo_filter_list_meta_download_parts( $list = array() ) {
		$list['download'] = esc_html__( 'Downloads', 'fribbo' );
		return $list;
	}
	add_filter( 'fribbo_filter_list_meta_parts', 'fribbo_filter_list_meta_download_parts', 10 );


	if ( ! function_exists( 'fribbo_get_post_download_count' ) ) {
		function fribbo_get_post_download_count( $post_text = '', $src = true ) {
			global $post;
			$sum = 0;
			$rez = array();
			$unique = array();
			if ( empty( $post_text ) ) {
				$post_text = $post->post_content;
			}
			if( $pattern = preg_match_all( '/(\[download)[\w\s\"\=\,]+[\]]{1}/', $post_text, $matches ) ) {
				$matches1 = array();
				foreach ($matches[0] as $k => $v) {
					$matches1[] = fribbo_get_tag_attrib($v, '[download]', 'id');
				}
				$unique = array_unique($matches1);
			}

			if($unique) {
				foreach ($unique as $k => $v) {
					$count = (int) get_metadata( 'post', $v, '_download_count', true );
					$rez[] = $count;
					$sum += $count;
				}
			}
			return $sum;
		}
	}

	function fribbo_action_show_post_meta_download($comp, $id, $args ) {
		if ( 'download' == $comp ) {
			$post_content = fribbo_get_post_content();
			$post_download = fribbo_get_post_download_count( $post_content );
			echo ( is_single()
					? '<span'
					: '<a href="' . esc_url( get_permalink() ) . '"'
				)
				. ' class="post_meta_item post_meta_download">'
				. '<span class="post_meta_number">' . esc_html( $post_download ) . '</span>'
				. ( is_single()
					? '</span>'
					: '</a>'
				);
		}
	}
	add_action( 'fribbo_action_show_post_meta', 'fribbo_action_show_post_meta_download', 100, 3 );

}