<?php
/**
 * Skins support: Main skin file for the skin 'Default'
 *
 * Load scripts and styles,
 * and other operations that affect the appearance and behavior of the theme
 * when the skin is activated
 *
 * @package FRIBBO
 * @since FRIBBO 1.0.46
 */



require_once FRIBBO_THEME_DIR . fribbo_skins_get_current_skin_dir() . 'plugins/download-monitor/download-monitor.php';


// SKIN SETUP
//--------------------------------------------------------------------

// Theme init priorities:
// 1 - register filters to add/remove lists items in the Theme Options
if ( ! function_exists( 'fribbo_skin_theme_setup1' ) ) {
	add_action( 'after_setup_theme', 'fribbo_skin_theme_setup1', 1 );
	function fribbo_skin_theme_setup1() {
		// ToDo: Set default values for typography, color schemes, etc.

		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		// !!! START SECTION: If skins are not used - move this section to the theme-setup.php !!!
		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

		// Theme fonts: Google and/or custom fonts
		//--------------------------------------------
		// Fonts to load when theme start
		// It can be Google fonts or uploaded fonts, placed in the folder css/font-face/font-name inside the skin folder
		// Attention! Font's folder must have name equal to the font's name, with spaces replaced on the dash '-'
		// example: font name 'TeX Gyre Termes', folder 'TeX-Gyre-Termes'
		fribbo_storage_set(
			'load_fonts', array(
				// Google font
				array(
					'name'   => 'Rubik',
					'family' => 'sans-serif',
					'styles' => '300,300i,400,400i,500,500i,700,700i,900,900i',     // Parameter 'style' used only for the Google fonts
				),
			)
		);

		// Characters subset for the Google fonts. Available values are: latin,latin-ext,cyrillic,cyrillic-ext,greek,greek-ext,vietnamese
		fribbo_storage_set( 'load_fonts_subset', 'latin,latin-ext' );

		// Settings of the main tags
		// Attention! Font name in the parameter 'font-family' will be enclosed in the quotes and no spaces after comma!
		// example:
		// Correct:   'font-family' => '"Rubik",sans-serif'
		// Incorrect: 'font-family' => '"Rubik", sans-serif'
		// Incorrect: 'font-family' => 'Rubik,sans-serif'

		$font_description = esc_html__( 'Font settings of the %s of the site. To correctly display the site on mobile devices, use only the following units: "rem", "em" or "ex"', 'fribbo' );

		fribbo_storage_set(
			'theme_fonts', array(
				'p'       => array(
					'title'           => esc_html__( 'Main text', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'main text', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '1rem',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.52em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '',
					'margin-top'      => '0em',
					'margin-bottom'   => '1.2em',
				),
				'h1'      => array(
					'title'           => esc_html__( 'Heading 1', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'tag H1', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '3.125em',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.2em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
					'margin-top'      => '1.65em',
					'margin-bottom'   => '0.86em',
				),
				'h2'      => array(
					'title'           => esc_html__( 'Heading 2', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'tag H2', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '2.5em',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.25em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
					'margin-top'      => '1.88em',
					'margin-bottom'   => '0.98em',
				),
				'h3'      => array(
					'title'           => esc_html__( 'Heading 3', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'tag H3', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '2.188em',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.286em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
					'margin-top'      => '1.96em',
					'margin-bottom'   => '1.03em',
				),
				'h4'      => array(
					'title'           => esc_html__( 'Heading 4', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'tag H4', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '1.875em',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.333em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
					'margin-top'      => '2.1em',
					'margin-bottom'   => '1.04em',
				),
				'h5'      => array(
					'title'           => esc_html__( 'Heading 5', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'tag H5', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '1.5em',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.417em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
					'margin-top'      => '2.4em',
					'margin-bottom'   => '1.13em',
				),
				'h6'      => array(
					'title'           => esc_html__( 'Heading 6', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'tag H6', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '1.125em',
					'font-weight'     => '500',
					'font-style'      => 'normal',
					'line-height'     => '1.444em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
					'margin-top'      => '2.8em',
					'margin-bottom'   => '1.35em',
				),
				'logo'    => array(
					'title'           => esc_html__( 'Logo text', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'text of the logo', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '1.8em',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.25em',
					'text-decoration' => 'none',
					'text-transform'  => 'uppercase',
					'letter-spacing'  => '0px',
				),
				'button'  => array(
					'title'           => esc_html__( 'Buttons', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'buttons', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '16px',
					'font-weight'     => '500',
					'font-style'      => 'normal',
					'line-height'     => '20px',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
				),
				'input'   => array(
					'title'           => esc_html__( 'Input fields', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'input fields, dropdowns and textareas', 'fribbo' ) ),
					'font-family'     => 'inherit',
					'font-size'       => '1em',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.5em',     // Attention! Firefox don't allow line-height less then 1.5em in the select
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
				),
				'info'    => array(
					'title'           => esc_html__( 'Post meta', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'post meta (author, categories, publish date, counters, share, etc.)', 'fribbo' ) ),
					'font-family'     => 'inherit',
					'font-size'       => '12px',  // Old value '13px' don't allow using 'font zoom' in the custom blog items
					'font-weight'     => '500',
					'font-style'      => 'normal',
					'line-height'     => '14px',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
					'margin-top'      => '0px',
					'margin-bottom'   => '',
				),
				'menu'    => array(
					'title'           => esc_html__( 'Main menu', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'main menu items', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '16px',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.4em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
				),
				'submenu' => array(
					'title'           => esc_html__( 'Dropdown menu', 'fribbo' ),
					'description'     => sprintf( $font_description, esc_html__( 'dropdown menu items', 'fribbo' ) ),
					'font-family'     => '"Rubik",sans-serif',
					'font-size'       => '14px',
					'font-weight'     => '400',
					'font-style'      => 'normal',
					'line-height'     => '1.4em',
					'text-decoration' => 'none',
					'text-transform'  => 'none',
					'letter-spacing'  => '0px',
				),
			)
		);
		

		// Color schemes: default values
		//--------------------------------------------
		$schemes = array(

			// Color scheme: 'default'
			'default' => array(
				'title'    => esc_html__( 'Default', 'fribbo' ),
				'internal' => true,
				'colors'   => array(

					// Whole block border and background
					'bg_color'         => '#f5f7f7', //ok
					'bd_color'         => '#e5e8e8', //ok

					// Text and links colors
					'text'             => '#858787', //ok
					'text_light'       => '#5f5f5f', //ok ?
					'text_dark'        => '#1D2020', //ok
					'text_link'        => '#f75389', //ok
					'text_hover'       => '#1D2020', //ok
					'text_link2'       => '#0dbfbf', //ok
					'text_hover2'      => '#1D2020', //ok
					'text_link3'       => '#ddb837',
					'text_hover3'      => '#eec432',

					// Alternative blocks (sidebar, tabs, alternative blocks, etc.)
					'alter_bg_color'   => '#ffffff', //ok
					'alter_bg_hover'   => '#f5f7f7', //ok ?
					'alter_bd_color'   => '#e5e8e8', //ok
					'alter_bd_hover'   => '#d2dcd9', //ok
					'alter_text'       => '#858787', //ok
					'alter_light'      => '#b7b7b7',
					'alter_dark'       => '#1D2020', //ok
					'alter_link'       => '#f75389', //ok
					'alter_hover'      => '#1D2020', //ok
					'alter_link2'      => '#0dbfbf', //ok
					'alter_hover2'     => '#1D2020', //ok
					'alter_link3'      => '#eec432',
					'alter_hover3'     => '#ddb837',

					// Extra blocks (submenu, tabs, color blocks, etc.)
					'extra_bg_color'   => '#ffffff', //ok
					'extra_bg_hover'   => '#28272e',
					'extra_bd_color'   => '#313131',
					'extra_bd_hover'   => '#3d3d3d',
					'extra_text'       => '#858787', //ok
					'extra_light'      => '#afafaf',
					'extra_dark'       => '#1D2020', //ok
					'extra_link'       => '#f75389', //ok
					'extra_hover'      => '#1D2020', //ok
					'extra_link2'      => '#80d572',
					'extra_hover2'     => '#8be77c',
					'extra_link3'      => '#ddb837',
					'extra_hover3'     => '#eec432',

					// Input fields (form's fields and textarea)
					'input_bg_color'   => '#ffffff', //ok
					'input_bg_hover'   => '#ffffff', //ok
					'input_bd_color'   => '#ffffff', //ok
					'input_bd_hover'   => '#e5e8e8', //ok
					'input_text'       => '#1D2020', //ok
					'input_light'      => '#1D2020', //ok
					'input_dark'       => '#1D2020', //ok

					// Inverse blocks (text and links on the 'text_link' background)
					'inverse_bd_color' => '#67bcc1',
					'inverse_bd_hover' => '#5aa4a9',
					'inverse_text'     => '#1d1d1d',
					'inverse_light'    => '#333333',
					'inverse_dark'     => '#1D2020', //ok
					'inverse_link'     => '#ffffff',
					'inverse_hover'    => '#ffffff', //ok

					// Additional (skin-specific) colors.
					// Attention! Set of colors must be equal in all color schemes.
					//---> For example:
					//---> 'new_color1'         => '#rrggbb',
					//---> 'alter_new_color1'   => '#rrggbb',
					//---> 'inverse_new_color1' => '#rrggbb',
				),
			),

			// Color scheme: 'dark'
			'dark'    => array(
				'title'    => esc_html__( 'Dark', 'fribbo' ),
				'internal' => true,
				'colors'   => array(

					// Whole block border and background
					'bg_color'         => '#0e0d12',
					'bd_color'         => '#2e2c33',

					// Text and links colors
					'text'             => '#b7b7b7',
					'text_light'       => '#6f6f6f',
					'text_dark'        => '#ffffff',
					'text_link'        => '#f75389', //ok
					'text_hover'       => '#ffffff',
					'text_link2'       => '#0dbfbf', //ok
					'text_hover2'      => '#ffffff', //ok
					'text_link3'       => '#ddb837',
					'text_hover3'      => '#eec432',

					// Alternative blocks (sidebar, tabs, alternative blocks, etc.)
					'alter_bg_color'   => '#1e1d22',
					'alter_bg_hover'   => '#333333',
					'alter_bd_color'   => '#464646',
					'alter_bd_hover'   => '#4a4a4a',
					'alter_text'       => '#a6a6a6',
					'alter_light'      => '#6f6f6f',
					'alter_dark'       => '#ffffff',
					'alter_link'       => '#f75389', //ok
					'alter_hover'      => '#ffffff', //ok
					'alter_link2'      => '#0dbfbf', //ok
					'alter_hover2'     => '#ffffff', //ok
					'alter_link3'      => '#eec432',
					'alter_hover3'     => '#ddb837',

					// Extra blocks (submenu, tabs, color blocks, etc.)
					'extra_bg_color'   => '#1e1d22',
					'extra_bg_hover'   => '#28272e',
					'extra_bd_color'   => '#464646',
					'extra_bd_hover'   => '#4a4a4a',
					'extra_text'       => '#a6a6a6',
					'extra_light'      => '#6f6f6f',
					'extra_dark'       => '#ffffff',
					'extra_link'       => '#f75389', //ok
					'extra_hover'      => '#ffffff', //ok
					'extra_link2'      => '#80d572',
					'extra_hover2'     => '#8be77c',
					'extra_link3'      => '#ddb837',
					'extra_hover3'     => '#eec432',

					// Input fields (form's fields and textarea)
					'input_bg_color'   => '#2e2d32',
					'input_bg_hover'   => '#2e2d32',
					'input_bd_color'   => '#2e2d32',
					'input_bd_hover'   => '#353535',
					'input_text'       => '#b7b7b7',
					'input_light'      => '#6f6f6f',
					'input_dark'       => '#ffffff',

					// Inverse blocks (text and links on the 'text_link' background)
					'inverse_bd_color' => '#e36650',
					'inverse_bd_hover' => '#cb5b47',
					'inverse_text'     => '#1d1d1d',
					'inverse_light'    => '#6f6f6f',
					'inverse_dark'     => '#1D2020', //ok
					'inverse_link'     => '#ffffff',
					'inverse_hover'    => '#1D2020', //ok

					// Additional (skin-specific) colors.
					// Attention! Set of colors must be equal in all color schemes.
					//---> For example:
					//---> 'new_color1'         => '#rrggbb',
					//---> 'alter_new_color1'   => '#rrggbb',
					//---> 'inverse_new_color1' => '#rrggbb',
				),
			),
		);
		fribbo_storage_set( 'schemes', $schemes );
		fribbo_storage_set( 'schemes_original', $schemes );

		// Add names of additional colors
		//---> For example:
		//---> fribbo_storage_set_array( 'scheme_color_names', 'new_color1', array(
		//---> 	'title'       => __( 'New color 1', 'fribbo' ),
		//---> 	'description' => __( 'Description of the new color 1', 'fribbo' ),
		//---> ) );


		// Additional colors for each scheme
		// Parameters:	'color' - name of the color from the scheme that should be used as source for the transformation
		//				'alpha' - to make color transparent (0.0 - 1.0)
		//				'hue', 'saturation', 'brightness' - inc/dec value for each color's component
		fribbo_storage_set(
			'scheme_colors_add', array(
				'bg_color_0'        => array(
					'color' => 'bg_color',
					'alpha' => 0,
				),
				'bg_color_02'       => array(
					'color' => 'bg_color',
					'alpha' => 0.2,
				),
				'bg_color_07'       => array(
					'color' => 'bg_color',
					'alpha' => 0.7,
				),
				'bg_color_08'       => array(
					'color' => 'bg_color',
					'alpha' => 0.8,
				),
				'bg_color_09'       => array(
					'color' => 'bg_color',
					'alpha' => 0.9,
				),
				'alter_bg_color_07' => array(
					'color' => 'alter_bg_color',
					'alpha' => 0.7,
				),
				'alter_bg_color_04' => array(
					'color' => 'alter_bg_color',
					'alpha' => 0.4,
				),
				'alter_bg_color_00' => array(
					'color' => 'alter_bg_color',
					'alpha' => 0,
				),
				'alter_bg_color_02' => array(
					'color' => 'alter_bg_color',
					'alpha' => 0.2,
				),
				'alter_bd_color_02' => array(
					'color' => 'alter_bd_color',
					'alpha' => 0.2,
				),
				'alter_link_02'     => array(
					'color' => 'alter_link',
					'alpha' => 0.2,
				),
				'alter_link_07'     => array(
					'color' => 'alter_link',
					'alpha' => 0.7,
				),
				'extra_bg_color_02' => array(
					'color' => 'extra_bg_color',
					'alpha' => 0.2,
				),
				'extra_bg_color_05' => array(
					'color' => 'extra_bg_color',
					'alpha' => 0.5,
				),
				'extra_bg_color_07' => array(
					'color' => 'extra_bg_color',
					'alpha' => 0.7,
				),
				'extra_link_02'     => array(
					'color' => 'extra_link',
					'alpha' => 0.2,
				),
				'extra_link_07'     => array(
					'color' => 'extra_link',
					'alpha' => 0.7,
				),
				'text_dark_01'      => array(
					'color' => 'text_dark',
					'alpha' => 0.1,
				),
				'text_dark_02'      => array(
					'color' => 'text_dark',
					'alpha' => 0.2,
				),
				'text_dark_07'      => array(
					'color' => 'text_dark',
					'alpha' => 0.7,
				),
				'text_link_02'      => array(
					'color' => 'text_link',
					'alpha' => 0.2,
				),
				'text_link_07'      => array(
					'color' => 'text_link',
					'alpha' => 0.7,
				),
				'text_link_blend'   => array(
					'color'      => 'text_link',
					'hue'        => 2,
					'saturation' => -5,
					'brightness' => 5,
				),
				'alter_link_blend'  => array(
					'color'      => 'alter_link',
					'hue'        => 2,
					'saturation' => -5,
					'brightness' => 5,
				),
			)
		);

		// Theme specific thumb sizes
		//--------------------------------------------
		fribbo_storage_set(
			'theme_thumbs', apply_filters(
				'fribbo_filter_add_thumb_sizes', array(
					// Width of the image is equal to the content area width (without sidebar)
					// Height is fixed
					'fribbo-thumb-huge'        => array(
						'size'  => array( 1270, 714, true ),
						'title' => esc_html__( 'Huge image', 'fribbo' ),
						'subst' => 'trx_addons-thumb-huge',
					),
					// Width of the image is equal to the content area width (with sidebar)
					// Height is fixed
					'fribbo-thumb-big'         => array(
						'size'  => array( 787, 443, true ),
						'title' => esc_html__( 'Large image', 'fribbo' ),
						'subst' => 'trx_addons-thumb-big',
					),

					// Width of the image is equal to the 1/3 of the content area width (without sidebar)
					// Height is fixed
					'fribbo-thumb-med'         => array(
						'size'  => array( 404, 227, true ),
						'title' => esc_html__( 'Medium image', 'fribbo' ),
						'subst' => 'trx_addons-thumb-medium',
					),

					// Small square image (for avatars in comments, etc.)
					'fribbo-thumb-tiny'        => array(
						'size'  => array( 100, 100, true ),
						'title' => esc_html__( 'Small square avatar', 'fribbo' ),
						'subst' => 'trx_addons-thumb-tiny',
					),

					// Width of the image is equal to the content area width (with sidebar)
					// Height is proportional (only downscale, not crop)
					'fribbo-thumb-masonry-big' => array(
						'size'  => array( 787, 0, false ),     // Only downscale, not crop
						'title' => esc_html__( 'Masonry Large (scaled)', 'fribbo' ),
						'subst' => 'trx_addons-thumb-masonry-big',
					),

					// Width of the image is equal to the 1/3 of the full content area width (without sidebar)
					// Height is proportional (only downscale, not crop)
					'fribbo-thumb-masonry'     => array(
						'size'  => array( 424, 0, false ),     // Only downscale, not crop
						'title' => esc_html__( 'Masonry (scaled)', 'fribbo' ),
						'subst' => 'trx_addons-thumb-masonry',
					),

					'fribbo-thumb-avatar'        => array(
						'size'  => array( 600, 600, true ),
						'title' => esc_html__( 'Medium square avatar', 'fribbo' ),
						'subst' => 'trx_addons-thumb-avatar',
					),

					'fribbo-thumb-extra'        => array(
						'size'  => array( 630, 497, true ),
						'title' => esc_html__( 'Extra', 'fribbo' ),
						'subst' => 'trx_addons-thumb-extra',
					),
				)
			)
		);

		// Theme-specific blog layouts
		//----------------------------------------------
		$blog_styles = array(
			'excerpt' => array(
				'title'   => esc_html__( 'Standard', 'fribbo' ),
				'archive' => 'index',
				'item'    => 'templates/content-excerpt',
				'styles'  => 'excerpt',
				'icon'    => "images/theme-options/blog-style/excerpt.png",
			),
			'classic' => array(
				'title'   => esc_html__( 'Classic', 'fribbo' ),
				'archive' => 'index',
				'item'    => 'templates/content-classic',
				'columns' => array( 2, 3, 4 ),
				'styles'  => 'classic',
				'icon'    => "images/theme-options/blog-style/classic-%d.png",
				'new_row' => true,
			),
		);
		if ( ! FRIBBO_THEME_FREE ) {
			$blog_styles['classic-masonry']   = array(
				'title'   => esc_html__( 'Classic Masonry', 'fribbo' ),
				'archive' => 'index',
				'item'    => 'templates/content-classic',
				'columns' => array( 2, 3, 4 ),
				'styles'  => array( 'classic', 'masonry' ),
				'scripts' => 'masonry',
				'icon'    => "images/theme-options/blog-style/classic-masonry-%d.png",
				'new_row' => true,
			);
			$blog_styles['portfolio'] = array(
				'title'   => esc_html__( 'Portfolio', 'fribbo' ),
				'archive' => 'index',
				'item'    => 'templates/content-portfolio',
				'columns' => array( 2, 3, 4 ),
				'styles'  => 'portfolio',
				'icon'    => "images/theme-options/blog-style/portfolio-%d.png",
				'new_row' => true,
			);
			$blog_styles['portfolio-masonry'] = array(
				'title'   => esc_html__( 'Portfolio Masonry', 'fribbo' ),
				'archive' => 'index',
				'item'    => 'templates/content-portfolio',
				'columns' => array( 2, 3, 4 ),
				'styles'  => array( 'portfolio', 'masonry' ),
				'scripts' => 'masonry',
				'icon'    => "images/theme-options/blog-style/portfolio-masonry-%d.png",
				'new_row' => true,
			);
		}
		fribbo_storage_set( 'blog_styles', apply_filters( 'fribbo_filter_add_blog_styles', $blog_styles ) );


		// Theme-specific single post layouts
		//----------------------------------------------
		fribbo_storage_set( 'single_styles', apply_filters( 'fribbo_filter_add_single_styles', array(
			'in-over'    => array(
				'title'       => esc_html__( 'Standard 1', 'fribbo' ),
				'description' => esc_html__( 'The image inside the content area, the title over image', 'fribbo' ),
				'styles'      => 'in-over',
				'icon'        => "images/theme-options/single-style/in-over.png",
			),
			'in-sticky'  => array(
				'title'       => esc_html__( 'Standard 2', 'fribbo' ),
				'description' => esc_html__( 'The image inside the content area, the title is stick at the bottom side of the image', 'fribbo' ),
				'styles'      => 'in-sticky',
				'icon'        => "images/theme-options/single-style/in-sticky.png",
			),
			'in-below'   => array(
				'title'       => esc_html__( 'Standard 3', 'fribbo' ),
				'description' => esc_html__( 'The image inside the content area, the title below image', 'fribbo' ),
				'styles'      => 'in-below',
				'icon'        => "images/theme-options/single-style/in-below.png",
			),
			'in-above'   => array(
				'title'       => esc_html__( 'Standard 4', 'fribbo' ),
				'description' => esc_html__( 'The image inside the content area, the title above image', 'fribbo' ),
				'styles'      => 'in-above',
				'icon'        => "images/theme-options/single-style/in-above.png",
			),
			'out-over-boxed'   => array(
				'title'       => esc_html__( 'Boxed 1', 'fribbo' ),
				'description' => esc_html__( 'Boxed image above the content area, the title over image', 'fribbo' ),
				'styles'      => 'out-over-boxed',
				'new_row'     => true,
				'icon'        => "images/theme-options/single-style/out-over-boxed.png",
			),
			'out-sticky-boxed' => array(
				'title'       => esc_html__( 'Boxed 2', 'fribbo' ),
				'description' => esc_html__( 'Boxed image above the content area, the title is stick at the bottom side of the image', 'fribbo' ),
				'styles'      => 'out-sticky-boxed',
				'icon'        => "images/theme-options/single-style/out-sticky-boxed.png",
			),
			'out-below-boxed'  => array(
				'title'       => esc_html__( 'Boxed 3', 'fribbo' ),
				'description' => esc_html__( 'Boxed image above the content area, the title below image', 'fribbo' ),
				'styles'      => 'out-below-boxed',
				'icon'        => "images/theme-options/single-style/out-below-boxed.png",
			),
			'out-over-fullwidth'   => array(
				'title'       => esc_html__( 'Fullwidth 1', 'fribbo' ),
				'description' => esc_html__( 'Fullwidth image above the content area, the title over image', 'fribbo' ),
				'styles'      => 'out-over-fullwidth',
				'new_row'     => true,
				'icon'        => "images/theme-options/single-style/out-over-fullwidth.png",
			),
			'out-sticky-fullwidth' => array(
				'title'       => esc_html__( 'Fullwidth 2', 'fribbo' ),
				'description' => esc_html__( 'Fullwidth image above the content area, the title is stick at the bottom side of the image', 'fribbo' ),
				'styles'      => 'out-sticky-fullwidth',
				'icon'        => "images/theme-options/single-style/out-sticky-fullwidth.png",
			),
			'out-below-fullwidth'  => array(
				'title'       => esc_html__( 'Fullwidth 3', 'fribbo' ),
				'description' => esc_html__( 'Fullwidth image above the content area, the title below image', 'fribbo' ),
				'styles'      => 'out-below-fullwidth',
				'icon'        => "images/theme-options/single-style/out-below-fullwidth.png",
			),
		) ) );

		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		// !!! END SECTION: If skins are not used - move this section to the theme-setup.php !!!
		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}
}

// Set theme specific importer options
if ( ! function_exists( 'fribbo_skin_importer_set_options' ) ) {
	add_filter('trx_addons_filter_importer_options', 'fribbo_skin_importer_set_options', 9);
	function fribbo_skin_importer_set_options($options = array()) {
		if ( is_array( $options ) ) {
			// ToDo: Prepare dummy-data installer's options
			//---> For example:
			//---> $options['demo_type']                         = 'skin_slug';
			//---> $options['files']['skin_slug']                = $options['files']['default'];
			//---> $options['files']['skin_slug']['title']       = esc_html__('Default Demo', 'fribbo');
			//---> $options['files']['skin_slug']['domain_dev']  = esc_url( fribbo_get_protocol() . '://skin.domain.dev' );
			//---> $options['files']['skin_slug']['domain_demo'] = esc_url( fribbo_get_protocol() . '://skin.domain.demo' );
		}
		return $options;
	}
}


// Filter to add in the required plugins list
// Priority 11 to add new plugins to the end of the list
if ( ! function_exists( 'fribbo_skin_tgmpa_required_plugins' ) ) {
	add_filter( 'fribbo_filter_tgmpa_required_plugins', 'fribbo_skin_tgmpa_required_plugins', 11 );
	function fribbo_skin_tgmpa_required_plugins( $list = array() ) {
		// ToDo: Check if plugin is in the 'required_plugins' and add his parameters to the TGMPA-list
		//       Replace 'skin-specific-plugin-slug' to the real slug of the plugin
		if ( fribbo_storage_isset( 'required_plugins', 'skin-specific-plugin-slug' ) ) {
			$list[] = array(
				'name'     => fribbo_storage_get_array( 'required_plugins', 'skin-specific-plugin-slug', 'title' ),
				'slug'     => 'skin-specific-plugin-slug',
				'required' => false,
			);
		}
		return $list;
	}
}


// Theme init priorities:
// 3 - add/remove Theme Options elements
if ( ! function_exists( 'fribbo_skin_theme_setup3' ) ) {
	add_action( 'after_setup_theme', 'fribbo_skin_theme_setup3', 3 );
	function fribbo_skin_theme_setup3() {
		// ToDo: Add / Modify theme options, color schemes, required plugins, etc.

		fribbo_storage_set_array('required_plugins','download-monitor',
			array(
				'title'       => esc_html__( 'Download Monitor', 'fribbo' ),
				'description' => '',
				'required'    => false,
				'logo'        => 'download-monitor.png',
				'group'       => esc_html__( 'Other', 'fribbo' ),
			)
		);

	}
}

// Filter to add/remove components of ThemeREX Addons when current skin is active
if ( ! function_exists( 'fribbo_skin_trx_addons_default_components' ) ) {
	add_filter('trx_addons_filter_load_options', 'fribbo_skin_trx_addons_default_components', 20);
	function fribbo_skin_trx_addons_default_components($components) {
		// ToDo: Set key value in the array $components to 0 (disable component) or 1 (enable component)
		//---> For example (enable reviews for posts):
		//---> $components['components_components_reviews'] = 1;
		return $components;
	}
}

// Filter to add/remove CPT
if ( ! function_exists( 'fribbo_skin_trx_addons_cpt_list' ) ) {
	add_filter('trx_addons_cpt_list', 'fribbo_skin_trx_addons_cpt_list');
	function fribbo_skin_trx_addons_cpt_list( $list = array() ) {
		// ToDo: Unset CPT slug from list to disable CPT when current skin is active
		//---> For example to disable CPT 'Portfolio':
		//---> unset( $list['portfolio'] );
		return $list;
	}
}

// Filter to add/remove shortcodes
if ( ! function_exists( 'fribbo_skin_trx_addons_sc_list' ) ) {
	add_filter('trx_addons_sc_list', 'fribbo_skin_trx_addons_sc_list');
	function fribbo_skin_trx_addons_sc_list( $list = array() ) {
		// ToDo: Unset shortcode's slug from list to disable shortcode when current skin is active
		//---> For example to disable shortcode 'Action':
		//---> unset( $list['action'] );

		// Also can be used to add/remove/modify shortcodes params
		//---> For example to add new template to the 'Blogger':
		//---> $list['blogger']['templates']['default']['new_template_slug'] = array(
		//--->		'title' => __('Title of the new template', 'fribbo'),
		//--->		'layout' => array(
		//--->			'featured' => array(),
		//--->			'content' => array('meta_categories', 'title', 'excerpt', 'meta', 'readmore')
		//--->		)
		//---> );
		return $list;
	}
}

// Filter to add/remove widgets
if ( ! function_exists( 'fribbo_skin_trx_addons_widgets_list' ) ) {
	add_filter('trx_addons_widgets_list', 'fribbo_skin_trx_addons_widgets_list');
	function fribbo_skin_trx_addons_widgets_list( $list = array() ) {
		// ToDo: Unset widget's slug from list to disable widget when current skin is active
		//---> For example to disable widget 'About Me':
		//---> unset( $list['aboutme'] );
		return $list;
	}
}



// SCRIPTS AND STYLES
//--------------------------------------------------

// Enqueue skin-specific scripts
// Priority 1050 -  before main theme plugins-specific (1100)
if ( ! function_exists( 'fribbo_skin_frontend_scripts' ) ) {
	add_action( 'wp_enqueue_scripts', 'fribbo_skin_frontend_scripts', 1050 );
	function fribbo_skin_frontend_scripts() {
		$fribbo_url = fribbo_get_file_url( fribbo_skins_get_current_skin_dir() . 'css/style.css' );
		if ( '' != $fribbo_url ) {
			wp_enqueue_style( 'fribbo-skin-' . esc_attr( fribbo_skins_get_current_skin_name() ), $fribbo_url, array(), null );
		}
		$fribbo_url = fribbo_get_file_url( fribbo_skins_get_current_skin_dir() . 'skin.js' );
		if ( '' != $fribbo_url ) {
			wp_enqueue_script( 'fribbo-skin-' . esc_attr( fribbo_skins_get_current_skin_name() ), $fribbo_url, array( 'jquery' ), null, true );
		}
	}
}


// Return array with current system info
if ( ! function_exists( 'fribbo_trx_addons_get_sys_info' ) ) {
	add_filter( 'trx_addons_filter_get_sys_info', 'fribbo_trx_addons_get_sys_info', 10);
	function fribbo_trx_addons_get_sys_info( $arr ) {
		$arr['wp_version']  = array(
			'title' => __('WP version', 'fribbo'),
			'value' => get_bloginfo( 'version' ),
			'recommended' => '4.7',
		);
		return $arr;
	}
}


// Custom styles
$fribbo_style_path = fribbo_get_file_dir( fribbo_skins_get_current_skin_dir() . 'css/style.php' );
if ( ! empty( $fribbo_style_path ) ) {
	require_once $fribbo_style_path;
}


// Contact Form 7
add_filter( 'wpcf7_form_elements', 'fribbo_wpcf7_form_elements', 1 );

if ( ! function_exists('fribbo_wpcf7_form_elements') ) {
	function fribbo_wpcf7_form_elements( $form ) {
		$reg_patern = '/\<input.*type=["|\']file["|\'].*?\>/';
		$match_result = preg_match( $reg_patern, $form, $match );

		if ( $match_result ) {
			$new_file_input = sprintf(
				'<span class="fileform"><span class="selectbutton">%1$s</span><span id="fileformlabel">%2$s</span>%3$s</span>',
				esc_html__( 'Select File', 'fribbo' ),
				esc_html__( 'Upload File', 'fribbo' ),
				$match[0]
			);
			$form = preg_replace( $reg_patern, $new_file_input, $form );
		}

		return $form;
	}
}


if ( ! function_exists( 'fribbo_elm_add_theme_animations' ) ) {
	add_filter( 'elementor/controls/animations/additional_animations', 'fribbo_elm_add_theme_animations' );
	function fribbo_elm_add_theme_animations( $animations ) {
		return array_merge( $animations, array(
			esc_html__( 'Theme Specific', 'fribbo' ) => array(
				'fribbo-fadeinup' => esc_html__( 'Fribbo - Fade In Up', 'fribbo' )
			)
		) );
	}
}