<?php
/**
 * The Classic template to display the content
 *
 * Used for index/archive/search.
 *
 * @package FRIBBO
 * @since FRIBBO 1.0
 */

$fribbo_template_args = get_query_var( 'fribbo_template_args' );

if ( is_array( $fribbo_template_args ) ) {
	$fribbo_columns    = empty( $fribbo_template_args['columns'] ) ? 2 : max( 1, $fribbo_template_args['columns'] );
	$fribbo_blog_style = array( $fribbo_template_args['type'], $fribbo_columns );
} else {
	$fribbo_blog_style = explode( '_', fribbo_get_theme_option( 'blog_style' ) );
	$fribbo_columns    = empty( $fribbo_blog_style[1] ) ? 2 : max( 1, $fribbo_blog_style[1] );
}
$fribbo_expanded   = ! fribbo_sidebar_present() && fribbo_get_theme_option( 'expand_content' ) == 'expand';

$fribbo_post_format = get_post_format();
$fribbo_post_format = empty( $fribbo_post_format ) ? 'standard' : str_replace( 'post-format-', '', $fribbo_post_format );

?><div class="<?php
	if ( ! empty( $fribbo_template_args['slider'] ) ) {
		echo ' slider-slide swiper-slide';
	} else {
		echo ( fribbo_is_blog_style_use_masonry( $fribbo_blog_style[0] ) ? 'masonry_item masonry_item' : 'column' ) . '-1_' . esc_attr( $fribbo_columns );
	}
?>"><article id="post-<?php the_ID(); ?>" data-post-id="<?php the_ID(); ?>"
	<?php
	post_class(
		'post_item post_item_container post_format_' . esc_attr( $fribbo_post_format )
				. ' post_layout_classic post_layout_classic_' . esc_attr( $fribbo_columns )
				. ' post_layout_' . esc_attr( $fribbo_blog_style[0] )
				. ' post_layout_' . esc_attr( $fribbo_blog_style[0] ) . '_' . esc_attr( $fribbo_columns )
	);
	fribbo_add_blog_animation( $fribbo_template_args );
	?>
>
	<?php

	// Sticky label
	if ( is_sticky() && ! is_paged() ) {
		?>
		<span class="post_label label_sticky"></span>
		<?php
	}

	// Featured image
	$fribbo_hover      = ! empty( $fribbo_template_args['hover'] ) && ! fribbo_is_inherit( $fribbo_template_args['hover'] )
							? $fribbo_template_args['hover']
							: fribbo_get_theme_option( 'image_hover' );

	$fribbo_components = ! empty( $fribbo_template_args['meta_parts'] )
							? ( is_array( $fribbo_template_args['meta_parts'] )
								? $fribbo_template_args['meta_parts']
								: explode( ',', $fribbo_template_args['meta_parts'] )
								)
							: fribbo_array_get_keys_by_value( fribbo_get_theme_option( 'meta_parts' ) );

	fribbo_show_post_featured(
		array(
			'thumb_size' => fribbo_get_thumb_size(
				'classic' == $fribbo_blog_style[0]
						? ( strpos( fribbo_get_theme_option( 'body_style' ), 'full' ) !== false
								? ( $fribbo_columns > 2 ? 'big' : 'huge' )
								: ( $fribbo_columns > 2
									? ( $fribbo_expanded ? 'med' : 'small' )
									: ( $fribbo_expanded ? 'big' : 'med' )
									)
							)
						: ( strpos( fribbo_get_theme_option( 'body_style' ), 'full' ) !== false
								? ( $fribbo_columns > 2 ? 'masonry-big' : 'full' )
								: ( $fribbo_columns <= 2 && $fribbo_expanded ? 'masonry-big' : 'masonry' )
							)
			),
			'hover'      => $fribbo_hover,
			'meta_parts' => $fribbo_components,
			'no_links'   => ! empty( $fribbo_template_args['no_links'] ),
		)
	);

	// Title and post meta
	$fribbo_show_title = get_the_title() != '';
	$fribbo_show_meta  = count( $fribbo_components ) > 0 && ! in_array( $fribbo_hover, array( 'border', 'pull', 'slide', 'fade', 'info' ) );

	if ( $fribbo_show_title ) {
		?>
		<div class="post_header entry-header">
			<?php
			// Post title
			if ( apply_filters( 'fribbo_filter_show_blog_title', true, 'classic' ) ) {
				do_action( 'fribbo_action_before_post_title' );
				if ( empty( $fribbo_template_args['no_links'] ) ) {
					the_title( sprintf( '<h4 class="post_title entry-title"><a href="%s" rel="bookmark">', esc_url( get_permalink() ) ), '</a></h4>' );
				} else {
					the_title( '<h4 class="post_title entry-title">', '</h4>' );
				}
				do_action( 'fribbo_action_after_post_title' );
			}

			// Post meta
			if ( apply_filters( 'fribbo_filter_show_blog_meta', $fribbo_show_meta, $fribbo_components, 'classic' ) ) {
				if ( count( $fribbo_components ) > 0 ) {
					do_action( 'fribbo_action_before_post_meta' );
					fribbo_show_post_meta(
						apply_filters(
							'fribbo_filter_post_meta_args', array(
							'components' => join( ',', $fribbo_components ),
							'seo'        => false,
							'echo'       => true,
						), $fribbo_blog_style[0], $fribbo_columns
						)
					);
					do_action( 'fribbo_action_after_post_meta' );
				}
			}

			?>
		</div><!-- .entry-header -->
		<?php
	}

	// Post content
	ob_start();
	if ( apply_filters( 'fribbo_filter_show_blog_excerpt', empty( $fribbo_template_args['hide_excerpt'] ) && fribbo_get_theme_option( 'excerpt_length' ) > 0, 'classic' ) ) {
		fribbo_show_post_content( $fribbo_template_args, '<div class="post_content_inner">', '</div>' );
	}
	$fribbo_content = ob_get_contents();
	ob_end_clean();

	fribbo_show_layout( $fribbo_content, '<div class="post_content entry-content">', '</div><!-- .entry-content -->' );
		
	// More button
	if ( apply_filters( 'fribbo_filter_show_blog_readmore', ! $fribbo_show_title, 'classic' ) ) {
		if ( empty( $fribbo_template_args['no_links'] ) ) {
			do_action( 'fribbo_action_before_post_readmore' );
			fribbo_show_post_more_link( $fribbo_template_args, '<p>', '</p>' );
			do_action( 'fribbo_action_after_post_readmore' );
		}
	}
	?>

</article></div><?php
// Need opening PHP-tag above, because <div> is a inline-block element (used as column)!
