<?php
/**
 * The default template to display the content
 *
 * Used for index/archive/search.
 *
 * @package FRIBBO
 * @since FRIBBO 1.0
 */

$fribbo_template_args = get_query_var( 'fribbo_template_args' );
$fribbo_columns = 1;
if ( is_array( $fribbo_template_args ) ) {
	$fribbo_columns    = empty( $fribbo_template_args['columns'] ) ? 1 : max( 1, $fribbo_template_args['columns'] );
	$fribbo_blog_style = array( $fribbo_template_args['type'], $fribbo_columns );
	if ( ! empty( $fribbo_template_args['slider'] ) ) {
		?><div class="slider-slide swiper-slide">
		<?php
	} elseif ( $fribbo_columns > 1 ) {
		?>
		<div class="column-1_<?php echo esc_attr( $fribbo_columns ); ?>">
		<?php
	}
}
$fribbo_expanded    = ! fribbo_sidebar_present() && fribbo_get_theme_option( 'expand_content' ) == 'expand';
$fribbo_post_format = get_post_format();
$fribbo_post_format = empty( $fribbo_post_format ) ? 'standard' : str_replace( 'post-format-', '', $fribbo_post_format );
?>
<article id="post-<?php the_ID(); ?>" data-post-id="<?php the_ID(); ?>"
	<?php
	post_class( 'post_item post_item_container post_layout_excerpt post_format_' . esc_attr( $fribbo_post_format ) );
	fribbo_add_blog_animation( $fribbo_template_args );
	?>
>
	<?php

	// Sticky label
	if ( is_sticky() && ! is_paged() ) {
		?>
		<span class="post_label label_sticky"></span>
		<?php
	}

	// Featured image
	$fribbo_hover      = ! empty( $fribbo_template_args['hover'] ) && ! fribbo_is_inherit( $fribbo_template_args['hover'] )
							? $fribbo_template_args['hover']
							: fribbo_get_theme_option( 'image_hover' );
	$fribbo_components = ! empty( $fribbo_template_args['meta_parts'] )
							? ( is_array( $fribbo_template_args['meta_parts'] )
								? $fribbo_template_args['meta_parts']
								: array_map( 'trim', explode( ',', $fribbo_template_args['meta_parts'] ) )
								)
							: fribbo_array_get_keys_by_value( fribbo_get_theme_option( 'meta_parts' ) );

	// Cat in top featured
	$categories_show = '';
	if ( ! in_array( get_post_format(), array( 'audio' ) ) && has_post_thumbnail() ) {
		if( in_array( $fribbo_hover, array('icon', 'icons') ) ) {
			ob_start();
			?>
			<div class="post_meta cat_top">
							<?php
							if ( in_array( 'categories', $fribbo_components ) ) {
								if ( apply_filters( 'fribbo_filter_show_blog_categories', true, array( 'categories' ) ) ) {
									?>
									<div class="post_category">
										<?php
										$categories = fribbo_show_post_meta(
																		array(
																			'components' => 'categories',
																			'seo'        => false,
																			'echo'       => false,
																			)

															);
										fribbo_show_layout(str_replace( ', ', '', $categories ));
										?>
									</div>
									<?php
								}
								$fribbo_components = fribbo_array_delete_by_value( $fribbo_components, 'categories' );
							}
							?>
			</div>
			<?php
			$categories_show = ob_get_contents();
			ob_end_clean();
		}
	}

	fribbo_show_post_featured(
		array(
			'post_info'   => !empty($categories_show) ? $categories_show : '',
			'no_links'   => ! empty( $fribbo_template_args['no_links'] ),
			'hover'      => $fribbo_hover,
			'meta_parts' => $fribbo_components,
			'thumb_size' => fribbo_get_thumb_size( strpos( fribbo_get_theme_option( 'body_style' ), 'full' ) !== false
								? 'full'
								: ( $fribbo_expanded
									? 'huge'
									: 'big'
									)
								),
		)
	);

	// Title and post meta
	$fribbo_show_title = get_the_title() != '';
	$fribbo_show_meta  = count( $fribbo_components ) > 0 && ! in_array( $fribbo_hover, array( 'border', 'pull', 'slide', 'fade', 'info' ) );

	if ( $fribbo_show_title ) {
		?>
		<div class="post_header entry-header">
			<?php
			// Post title
			if ( apply_filters( 'fribbo_filter_show_blog_title', true, 'excerpt' ) ) {
				do_action( 'fribbo_action_before_post_title' );
				if ( empty( $fribbo_template_args['no_links'] ) ) {
					the_title( sprintf( '<h3 class="post_title entry-title"><a href="%s" rel="bookmark">', esc_url( get_permalink() ) ), '</a></h3>' );
				} else {
					the_title( '<h3 class="post_title entry-title">', '</h3>' );
				}
				do_action( 'fribbo_action_after_post_title' );
			}
			?>
		</div><!-- .post_header -->
		<?php
	}

	// Post content
	if ( apply_filters( 'fribbo_filter_show_blog_excerpt', empty( $fribbo_template_args['hide_excerpt'] ) && fribbo_get_theme_option( 'excerpt_length' ) > 0, 'excerpt' ) ) {
		?>
		<div class="post_content entry-content">
			<?php

			// Post meta
			if ( apply_filters( 'fribbo_filter_show_blog_meta', $fribbo_show_meta, $fribbo_components, 'excerpt' ) ) {
				if ( count( $fribbo_components ) > 0 ) {
					do_action( 'fribbo_action_before_post_meta' );
					fribbo_show_post_meta(
						apply_filters(
							'fribbo_filter_post_meta_args', array(
								'components' => join( ',', $fribbo_components ),
								'seo'        => false,
								'echo'       => true,
							), 'excerpt', 1
						)
					);
					do_action( 'fribbo_action_after_post_meta' );
				}
			}

			if ( fribbo_get_theme_option( 'blog_content' ) == 'fullpost' ) {
				// Post content area
				?>
				<div class="post_content_inner">
					<?php
					do_action( 'fribbo_action_before_full_post_content' );
					the_content( '' );
					do_action( 'fribbo_action_after_full_post_content' );
					?>
				</div>
				<?php
				// Inner pages
				wp_link_pages(
					array(
						'before'      => '<div class="page_links"><span class="page_links_title">' . esc_html__( 'Pages:', 'fribbo' ) . '</span>',
						'after'       => '</div>',
						'link_before' => '<span>',
						'link_after'  => '</span>',
						'pagelink'    => '<span class="screen-reader-text">' . esc_html__( 'Page', 'fribbo' ) . ' </span>%',
						'separator'   => '<span class="screen-reader-text">, </span>',
					)
				);
			} else {
				// Post content area
				fribbo_show_post_content( $fribbo_template_args, '<div class="post_content_inner">', '</div>' );
			}

			if(in_array( get_post_format(), array( 'quote' ) )){
				$fribbo_template_args['no_links'] = true;
			}

			// More button
			if ( apply_filters( 'fribbo_filter_show_blog_readmore', true, 'excerpt' ) ) {
				if ( empty( $fribbo_template_args['no_links'] ) ) {
					do_action( 'fribbo_action_before_post_readmore' );
					if ( fribbo_get_theme_option( 'blog_content' ) != 'fullpost' ) {
						fribbo_show_post_more_link( $fribbo_template_args, '', '' );
					} else {
						fribbo_show_post_comments_link( $fribbo_template_args, '', '' );
					}
					do_action( 'fribbo_action_after_post_readmore' );
				}
			}
			?>
		</div><!-- .entry-content -->
		<?php
	}
	?>
</article>
<?php

if ( is_array( $fribbo_template_args ) ) {
	if ( ! empty( $fribbo_template_args['slider'] ) || $fribbo_columns > 1 ) {
		?>
		</div>
		<?php
	}
}
