/**
 * Init scripts
 *
 * @package ThemeREX Addons
 * @since v1.0
 */

/* global jQuery:false */
/* global TRX_ADDONS_STORAGE:false */

jQuery( document ).ready( function() {

	"use strict";

	var vc_init_counter = 0;

	var parallax_controller = null;

	var requestAnimationFrame = window.requestAnimationFrame
								|| window.webkitRequestAnimationFrame
								|| window.mozRequestAnimationFrame
								|| window.oRequestAnimationFrame
								|| window.msRequestAnimationFrame
								|| null;

	var mouseX = null, mouseY = null,
		destX = 0, destY = 0,
		mouse_helper_timeout = 0,
		mouse_helper_target = null,
		mouse_helper_action = '',
		mouse_helper_color = '',
		mouse_helper_bg_color = '',
		mouse_helper_bd_color = '',
		mouse_helper_callback = '',
		mouse_helper_axis = 'xy',
		mouse_helper_delay_default = 10,
		mouse_helper_delay = mouse_helper_delay_default;

	var _bg_canvas = false;

	var $window             = jQuery( window ),
		_window_height      = $window.height(),
		_window_width       = $window.width(),
		_window_scroll_top  = $window.scrollTop(),
		_window_scroll_left = $window.scrollLeft(),
		$document           = jQuery( document ),
		_document_height    = $document.height(),
		$html               = jQuery( 'html' ),
		$body               = jQuery( 'body' ),
		$adminbar           = jQuery('#wpadminbar'),
		_adminbar_height    = trx_addons_adminbar_height(),
		_fixed_rows_height  = trx_addons_fixed_rows_height(),
		$scroll_to_top      = jQuery('.trx_addons_scroll_to_top'),
		$mouse_helper       = jQuery('.trx_addons_mouse_helper'),
		$page_preloader     = jQuery('#page_preloader');

	var $show_on_scroll     = jQuery('.trx_addons_show_on_scroll'),
		$banner_placeholder = jQuery('.trx_addons_banner_placeholder'),
		$animated_elements  = jQuery('[data-animation^="animated"]:not(.animated)'),
		$animated_hover     = jQuery('[data-hover-animation^="animated"]:not(.animated)'),
		$video_sticky       = jQuery('.trx_addons_video_sticky'),
		_video_sticky_fade  = true,
		$fixed_columns      = jQuery('.sc_column_fixed'),
		$stack_sections     = jQuery('.sc_stack_section_on:not(.elementor-element-edit-mode)'),
		$parallax_wrap      = jQuery('.sc_parallax_wrap' );

	var $video_tags         = jQuery('video'),
		$iframe_tags        = jQuery('.video_frame iframe,iframe');

	// Update links and values after the new post added
	$document.on( 'action.got_ajax_response', update_jquery_links );
	$document.on( 'action.init_hidden_elements', update_jquery_links );
	function update_jquery_links() {
		$show_on_scroll     = jQuery('.trx_addons_show_on_scroll');
		$banner_placeholder = jQuery('.trx_addons_banner_placeholder');
		$animated_elements  = jQuery('[data-animation^="animated"]:not(.animated)');
		$animated_hover     = jQuery('[data-hover-animation^="animated"]:not(.animated)');
		$video_sticky       = jQuery('.trx_addons_video_sticky');
		$fixed_columns      = jQuery('.sc_column_fixed');
		$stack_sections     = jQuery('.sc_stack_section_on:not(.elementor-element-edit-mode)');
		$parallax_wrap      = jQuery('.sc_parallax_wrap' );
		$video_tags         = jQuery('video');
		$iframe_tags        = jQuery('.video_frame iframe,iframe');
	}

	// Update fixed rows height
	$document.on( 'action.sc_layouts_row_fixed_on', update_fixed_rows_height );
	$document.on( 'action.sc_layouts_row_fixed_off', update_fixed_rows_height );
	function update_fixed_rows_height() {
		_fixed_rows_height = trx_addons_fixed_rows_height();
		document.querySelector('html').style.setProperty( '--fixed-rows-height', _fixed_rows_height+'px' );
	}


	// Page preloader
	//-----------------------------------------

	// Show preloader
	$window.on( 'beforeunload', function(e) {
		if ( jQuery.browser && ! jQuery.browser.safari ) {
			if ( typeof e.srcElement == 'undefined'
				|| typeof e.srcElement.activeElement == 'undefined'
				|| typeof e.srcElement.activeElement.href == 'undefined'
				|| e.srcElement.activeElement.href.indexOf('//') == 0
				|| e.srcElement.activeElement.href.indexOf('http:') == 0
				|| e.srcElement.activeElement.href.indexOf('https:') == 0
			) {
				$page_preloader
					.css( {
						display: 'block',
						opacity: 0
					} )
					.animate( {
						opacity: 0.8
					}, 300);
				setTimeout( trx_addons_hide_preloader, 5000 );
			}
		}
	});

	// Hide preloader
	function trx_addons_hide_preloader() {
		$page_preloader.animate(
			{
				opacity: 0
			},
			800,
			function() {
				jQuery(this).css( { display: 'none' } );
			}
		);
	}

	// Disable preloader on the links with 'tel:' or 'mailto:' or similar (not 'http:' or 'https:')
	$document.on( 'click', 'a', function() {
		var href = jQuery(this).attr('href');
		if ( href !== undefined ) {
			if ( href.indexOf('//') !== 0 && href.indexOf('http:') !== 0 && href.indexOf('https:') !== 0 ) {
				setTimeout( function() {
					if ( $page_preloader.css( 'display' ) == 'block' ) {
						$page_preloader.css( {
							display: 'none',
							opacity: 0
						} );
					}
				}, 1 );
			}
		}
	} );


	// Init actions
	//--------------------------------------

	trx_addons_init_actions();

	function trx_addons_init_actions() {
		if (typeof TRX_ADDONS_STORAGE == 'undefined') {
			window.TRX_ADDONS_STORAGE = {
										'vc_edit_mode': 0
									};
		}
		if (TRX_ADDONS_STORAGE['vc_edit_mode'] > 0 && jQuery('.vc_empty-placeholder').length === 0 && vc_init_counter++ < 30) {
			setTimeout(trx_addons_init_actions, 200);
			return;
		}

		update_fixed_rows_height();

		// Hide preloader
		trx_addons_hide_preloader();
		
		// Show system message
		var msg = jQuery('.trx_addons_message_box_system'),
			msg_delay = 5000;
		if (msg.length > 0) {
			setTimeout(function() {
				msg.fadeIn().delay(msg_delay).fadeOut();
			}, 1000);
			var login = jQuery('.trx_addons_login_link');
			if (msg.hasClass('trx_addons_message_box_error') && login.length > 0) {
				setTimeout(function() {
					login.trigger('click');
				}, 2000+msg_delay);
			}
		}

		// Shift page down to display hash-link if menu is fixed
		// (except WooCommerce product page, because WooCommerce has own handler of the hash url)
		if (typeof TRX_ADDONS_STORAGE['animate_to_hash']=='undefined' && !$body.hasClass('single-product')) {
			TRX_ADDONS_STORAGE['animate_to_hash'] = true;
			setTimeout(function() {
				// Hack for MailChimp - use our scroll to form, because his method damage layouts in the Chrome
				if (window.mc4wp_forms_config && window.mc4wp_forms_config.submitted_form && window.mc4wp_forms_config.submitted_form.element_id) {
					trx_addons_document_animate_to(window.mc4wp_forms_config.submitted_form.element_id);
				
				// Shift page down on fixed rows height
				} else if ( location.hash !== '' && location.hash != '#' && location.hash.indexOf('/') == -1 ) {
					var obj = jQuery(location.hash);
					if (obj.length > 0) {
						var off = obj.offset().top;
						if (!isNaN(off) && ((_fixed_rows_height > 0 && off - _window_scroll_top < _fixed_rows_height + 60) || _window_scroll_top === 0)) {
							trx_addons_document_animate_to(off - _fixed_rows_height - 60);
						}
					}
				}
			}, 600);
		}
		
		// Check for Retina display
		trx_addons_set_cookie('trx_addons_is_retina', trx_addons_is_retina() ? 1 : 0);

		// On switch to mobile layout: hide hover animations
		$document.on( 'action.switch_to_mobile_layout', function() {
			jQuery('[data-hover-animation^="animated"]').each(function() {
				var $self = jQuery(this);
				var animation = $self.data('hover-animation');
				var animation_out = $self.data('animation-out');
				if (animation_out === undefined) animation_out = "none";
				$self.removeClass(animation + ' ' + animation_out);
			});
		});

		// Add ready actions to the hidden elements actions
		var first_call = true;
		$document
			.on('action.init_hidden_elements', function() {
				// Init core elements
				trx_addons_ready_actions();
				// Generate 'scroll' event after hidden elements are inited
				if ( ! first_call ) $window.trigger('scroll');
				first_call = false;
			})
			// First call to init core actions
			.trigger('action.init_hidden_elements', [$body]);

		// Add our handlers after the VC init
		var vc_js = false;
		$document.on('vc_js', function() {
			if ( ! vc_js )	{
				vc_js = true;
				trx_addons_add_handlers();
			}
		});

		// Add our handlers if VC is no activated
		setTimeout(function() {
			if ( ! vc_js )	{
				trx_addons_add_handlers();
			}
		}, 1);
		
		// Add our handlers
		function trx_addons_add_handlers() {		
			// Resize handlers
			trx_addons_resize_actions();
			$window.resize( function() {
				trx_addons_resize_actions();
			} );

			// Scroll handlers
			TRX_ADDONS_STORAGE['scroll_busy'] = true;
			trx_addons_scroll_actions();
			$window.scroll( function() {
				if ( requestAnimationFrame ) {
					if ( ! TRX_ADDONS_STORAGE['scroll_busy'] ) {
						TRX_ADDONS_STORAGE['scroll_busy'] = true;
						requestAnimationFrame( trx_addons_scroll_actions );
					}
				} else {
					TRX_ADDONS_STORAGE['scroll_busy'] = true;
					trx_addons_scroll_actions();
				}
			} );

			// Inject our code in the VC function wpb_prepare_tab_content()
			// to init our elements on the new VC tabs, tour and accordion activation
			typeof window.wpb_prepare_tab_content == "function"
				&& typeof window.wpb_prepare_tab_content_old == "undefined"
				&& (window.wpb_prepare_tab_content_old = window.wpb_prepare_tab_content)
				&& (window.wpb_prepare_tab_content = function(e, ui) {
					// Call ThemeREX Addons actions
					if (typeof ui.newPanel !== 'undefined' && ui.newPanel.length > 0) {
						$document.trigger('action.init_hidden_elements', [ui.newPanel]);
					} else if (typeof ui.panel !== 'undefined' && ui.panel.length > 0) {
						$document.trigger('action.init_hidden_elements', [ui.panel]);
					}
					// Call old VC handler
					window.wpb_prepare_tab_content_old(e, ui);
				} );
			// Inject our code in the VC function vc_accordionActivate()
			// to init our elements on the old VC accordion activation
			typeof window.vc_accordionActivate == "function"
				&& typeof window.vc_accordionActivate_old == "undefined"
				&& (window.vc_accordionActivate_old = window.vc_accordionActivate)
				&& (window.vc_accordionActivate = function(e, ui) {
					// Call ThemeREX Addons actions
					if (typeof ui.newPanel !== 'undefined' && ui.newPanel.length > 0) {
						$document.trigger('action.init_hidden_elements', [ui.newPanel]);
					} else if (typeof ui.panel !== 'undefined' && ui.panel.length > 0) {
						$document.trigger('action.init_hidden_elements', [ui.panel]);
					}
					// Call old VC handler
					window.vc_accordionActivate_old(e, ui);
				} );
		}

		// Init mouse helper
		trx_addons_mouse_helper_init();

		// Init bg canvas
		trx_addons_bg_canvas_init();
	}


	// Page first load actions
	//==============================================
	function trx_addons_ready_actions(e, container) {
	
		if (container === undefined) container = $body;

		// Animate to the page-inner links
		//----------------------------------------------
		if (TRX_ADDONS_STORAGE['animate_inner_links'] > 0 && !container.hasClass('animate_to_inited')) {
			container
				.addClass('animate_to_inited')
				.on('click', 'a', function(e) {
					var link_obj = jQuery(this);
					var link_parent = link_obj.parent();
					// Skip tabs and accordions
					if (link_parent.parent().hasClass('trx_addons_tabs_titles')	// trx_addons_tabs
						|| link_obj.hasClass('trx_addons_panel_link')			// sc_layouts_panel
						|| link_obj.hasClass('trx_addons_popup_link')			// sc_layouts_popup
						|| link_parent.hasClass('vc_tta-tab') 					// new VC tabs, old VC tabs, new VC tour
						|| link_obj.hasClass('vc_pagination-trigger')			// pagination in VC tabs
						|| link_obj.hasClass('ui-tabs-anchor') 					// old VC tour
						|| link_parent.hasClass('vc_tta-panel-title')			// new VC accordion
						|| link_parent.hasClass('wpb_accordion_header') 		// old VC accordion
						|| link_parent.parents('.wc-tabs').length > 0 			// WooCommerce tabs on the single product page
						|| link_parent.parents('ul[class*="tabs"]').length > 0 	// All other tabs
					) {
						return true;
					}
					var href = link_obj.attr('href');
					if (href == '#') return true;
					if (trx_addons_is_local_link(href)) {
						var pos = href.indexOf('#'),
							offset = 0;
						if (pos >= 0) {
							href = href.substr(pos);
							if (jQuery(href).length > 0) {
								trx_addons_document_animate_to(href);
								e.preventDefault();
								return false;
							}
						}
					}
				});
		}

		// Add parameter target="_blank" to all external links
		//----------------------------------------------
		if (TRX_ADDONS_STORAGE['add_target_blank'] > 0) {
			jQuery('a').filter(function() {
				return this.hostname && this.hostname !== location.hostname;
			}).each( function() {
				var link = jQuery(this),
					rel  = link.attr('rel');
				if ( link.attr('target') != '_blank' ) {
					link.attr('target', '_blank');
				}
				if ( ! rel || rel.indexOf('nofollow') == -1 ) {
					link.attr('rel', ( rel ? rel + ' ' : '' ) + 'nofollow');
				}
			});
		}

		// Hide empty figcaptions
		jQuery('figcaption').each( function() {
			var $self = jQuery(this);
			if ( $self.text() == '' ) {
				$self.hide();
			}
		});

		// Tabs
		//------------------------------------
		if (jQuery.ui && jQuery.ui.tabs) {
			var $tabs = container.find('.trx_addons_tabs:not(.inited)');
			if ( $tabs.length > 0) {
				$tabs.each( function () {
					var $self = jQuery(this);
					// Get initially opened tab
					var init = $self.data('active');
					if (isNaN(init)) {
						init = 0;
						var active = $self.find('> ul > li[data-active="true"]').eq(0);
						if (active.length > 0) {
							init = active.index();
							if (isNaN(init) || init < 0) init = 0;
						}
					} else {
						init = Math.max(0, init);
					}
					// Get disabled tabs
					var disabled = [];
					$self.find('> ul > li[data-disabled="true"]').each( function() {
						disabled.push(jQuery(this).index());
					});
					// Init tabs
					$self.addClass('inited').tabs({
						active: init,
						disabled: disabled,
						show: {
							effect: 'fadeIn',
							duration: 300
						},
						hide: {
							effect: 'fadeOut',
							duration: 300
						},
						create: function( event, ui ) {
							if (ui.panel.length > 0) {
								$document.trigger('action.init_hidden_elements', [ui.panel]);
							}
						},
						activate: function( event, ui ) {
							if (ui.newPanel.length > 0) {
								$document.trigger('action.init_hidden_elements', [ui.newPanel]);
								$window.trigger('resize');
								$window.trigger('scroll');
							}
						}
					});
				});
			}
		}
	
	
		// Accordion
		//------------------------------------
		if (jQuery.ui && jQuery.ui.accordion) {
			var $accordion = container.find('.trx_addons_accordion:not(.inited)');
			if ( $accordion.length > 0) {
				$accordion.each(function () {
					// Get headers selector
					var accordion = jQuery(this);
					var headers = accordion.data('headers');
					if (headers===undefined) headers = 'h5';
					// Get height style
					var height_style = accordion.data('height-style');
					if (height_style===undefined) height_style = 'content';
					// Get collapsible
					var collapsible = accordion.data('collapsible');
					if (collapsible===undefined) collapsible = false;
					// Get initially opened tab
					var init = accordion.data('active');
					var active = false;
					if (isNaN(init)) {
						init = 0;
						var active = accordion.find(headers+'[data-active="true"]').eq(0);
						if (active.length > 0) {
							while (!active.parent().hasClass('trx_addons_accordion')) {
								active = active.parent();
							}
							init = active.index();
							if (isNaN(init) || init < 0) init = 0;
						}
					} else {
						init = Math.max(0, init);
					}
					// Init accordion
					accordion.addClass('inited').accordion({
						active: init,
						collapsible: collapsible,
						header: headers,
						heightStyle: height_style,
						create: function( event, ui ) {
							if (ui.panel.length > 0) {
								$document.trigger('action.init_hidden_elements', [ui.panel]);
							} else if (active !== false && active.length > 0) {
								// If headers and panels wrapped into div
								active.find('>'+headers).trigger('click');
							}
						},
						activate: function( event, ui ) {
							if (ui.newPanel.length > 0) {
								$document.trigger('action.init_hidden_elements', [ui.newPanel]);
								$window.trigger('resize');
								$window.trigger('scroll');
							}
						}
					});
				});
			}
		}
	
	
		// Color Picker
		//----------------------------------------------------------------
		var cp = container.find('.trx_addons_color_selector:not(.inited)'),
			cp_created = false;
		if (cp.length > 0) {
			cp.addClass('inited').each( function() {
				var $self = jQuery(this);
				// Internal ColorPicker
				if ( $self.hasClass('iColorPicker') ) {
					if (!cp_created) {
						trx_addons_color_picker();
						cp_created = true;
					}
					trx_addons_change_field_colors($self);
					$self
						.on('focus', function (e) {
							trx_addons_color_picker_show(null, jQuery(this), function(fld, clr) {
								fld.val(clr).trigger('change');
								trx_addons_change_field_colors(fld);
							});
						})
						.on('change', function(e) {
							trx_addons_change_field_colors(jQuery(this));
						});
					
				// WP ColorPicker - Iris
				} else if (typeof jQuery.fn.wpColorPicker != 'undefined') {
					$self.wpColorPicker({
						// you can declare a default color here,
						// or in the data-default-color attribute on the input
						//defaultColor: false,
				
						// a callback to fire whenever the color changes to a valid color
						change: function(e, ui){
							jQuery(e.target).val(ui.color).trigger('change');
						},
				
						// a callback to fire when the input is emptied or an invalid color
						clear: function(e) {
							jQuery(e.target).prev().trigger('change')
						},
				
						// hide the color picker controls on load
						//hide: true,
				
						// show a group of common colors beneath the square
						// or, supply an array of colors to customize further
						//palettes: true
					});
				}
			});
		}
	
		// Change colors of the field
		function trx_addons_change_field_colors(fld) {
			var clr = fld.val(),
				hsb = trx_addons_hex2hsb(clr);
			fld.css({
				'backgroundColor': clr,
				'color': hsb['b'] < 70 ? '#fff' : '#000'
			});
		}


		// Range Slider
		//------------------------------------
		if (jQuery.ui && jQuery.ui.slider) {
			var $range_slider = container.find('.trx_addons_range_slider:not(.inited)');
			if ($range_slider.length > 0) {
				$range_slider.each( function () {
					// Get parameters
					var range_slider = jQuery(this);
					var linked_field = range_slider.data('linked_field');
					if (linked_field===undefined) linked_field = range_slider.prev('input[type="hidden"]');
					else linked_field = jQuery('#'+linked_field);
					if (linked_field.length == 0) return;
					var range_slider_cur = range_slider.find('> .trx_addons_range_slider_label_cur');
					var range_slider_type = range_slider.data('range');
					if (range_slider_type===undefined) range_slider_type = 'min';
					var values = linked_field.val().split(',');
					var minimum = range_slider.data('min');
					if (minimum===undefined) minimum = 0;
					var maximum = range_slider.data('max');
					if (maximum===undefined) maximum = 0;
					var step = range_slider.data('step');
					if (step===undefined) step = 1;
					// Init range slider
					var init_obj = {
						range: range_slider_type,
						min: minimum,
						max: maximum,
						step: step,
						slide: function(event, ui) {
							var cur_values = range_slider_type === 'min' ? [ui.value] : ui.values;
							linked_field.val(cur_values.join(',')).trigger('change');
							for (var i=0; i < cur_values.length; i++) {
								range_slider_cur.eq(i)
										.html(cur_values[i])
										.css('left', Math.max(0, Math.min(100, (cur_values[i]-minimum)*100/(maximum-minimum)))+'%');
							}
						},
						create: function(event, ui) {
							for (var i=0; i < values.length; i++) {
								range_slider_cur.eq(i)
										.html(values[i])
										.css('left', Math.max(0, Math.min(100, (values[i]-minimum)*100/(maximum-minimum)))+'%');
							}
						}
					};
					if (range_slider_type === true)
						init_obj.values = values;
					else
						init_obj.value = values[0];
					range_slider.addClass('inited').slider(init_obj);
				});
			}
		}
	
	
		// Select2
		//------------------------------------
		if (jQuery.fn && jQuery.fn.select2) {
			container.find('.trx_addons_select2:not(.inited)').addClass('inited').select2();
		}
		
	
		// Video player
		//----------------------------------------------

		// Video frame 'Play' button
		var $video_hover = container.find('.trx_addons_video_player.with_cover .video_hover:not(.inited)');
		if ($video_hover.length > 0) {
			$video_hover
				.addClass('inited')
				.on('click', function(e) {
					var $self = jQuery(this);

					// If video in the popup
					if ($self.hasClass('trx_addons_popup_link')) return;
					
					$self.parents('.trx_addons_video_player').eq(0)
						.addClass('video_play')
						.find('.video_embed').html($self.data('video'));
	
					// If video in the slide
					var slider = $self.parents('.slider_swiper').eq(0);
					if (slider.length > 0) {
						var id = slider.attr('id');
						if ( typeof TRX_ADDONS_STORAGE['swipers'][id].autoplay != 'undefined' ) {
							TRX_ADDONS_STORAGE['swipers'][id].autoplay.stop();
							// If slider have controller - stop it too
							id = slider.data('controller');
							if (id && TRX_ADDONS_STORAGE['swipers'][id+'_swiper']) {
								TRX_ADDONS_STORAGE['swipers'][id+'_swiper'].autoplay.stop();
							}
						}
					}
	
					e.preventDefault();
					
					$document.trigger('action.init_hidden_elements', [$self.parents('.trx_addons_video_player').eq(0)]);
					$window.trigger('resize');
					
					return false;
				})
				.parents('.trx_addons_video_player')
				.on( 'click', function(e) {
					var $self = jQuery(this);
					if ( ! $self.hasClass('video_play') ) {
						jQuery(this).find('.video_hover').trigger('click');
						e.preventDefault();
						return false;
					}
				} );				
		}

		// Video player controller
		var $video_controller = container.find('.trx_addons_video_list_controller_wrap:not(.inited)');
		if ($video_controller.length > 0) {
			$video_controller
				.addClass('inited')
				.on('click', '.trx_addons_video_list_controller_item > a', function(e) {

					e.preventDefault();

					var item = jQuery(this).parent(),
						video = item.data('video'),
						title = item.data('title'),
						video_wrap = item.parents('.trx_addons_video_list').find('.trx_addons_video_list_video_wrap .trx_addons_video_player').parent();
					if ( ! item.hasClass('trx_addons_video_list_controller_item_active') && video && video_wrap.length == 1 ) {
						item.parent().find('.trx_addons_video_list_controller_item_active').removeClass('trx_addons_video_list_controller_item_active');
						item.addClass('trx_addons_video_list_controller_item_active');
						video_wrap.empty().append(video);
						if ( title ) {
							video_wrap.append(title);
						}
						$document.trigger('action.init_hidden_elements', [video_wrap]);
						$window.trigger('resize');
						if ( item.data( 'autoplay' ) > 0 && video_wrap.find( '.trx_addons_video_player .video_hover').length > 0 ) {
							video_wrap.find( '.trx_addons_video_player .video_hover').eq(0).trigger( 'click' );
						}
					}

					return false;
				});
		}

		// Video sticky - close button
		$video_sticky.each( function() {
			var video = jQuery(this);
			video
				.find( '.trx_addons_video_sticky_close:not(.inited)' )
				.addClass( 'inited' )
				.on( 'click', function( e ) {

					e.preventDefault();

					jQuery(this).hide();

					if ( _video_sticky_fade ) {
						video.addClass('trx_addons_video_sticky_on_fade').stop().animate({opacity:0}, 300, function() {
							video.parents('.post_featured').removeClass( 'with_video_sticky_on' );
							video.height('auto').removeClass( 'trx_addons_video_sticky trx_addons_video_sticky_on trx_addons_video_sticky_on_fade' ).stop().animate({'opacity': 1}, 500);
							$video_sticky = jQuery('.trx_addons_video_sticky');
							$window.trigger('resize');
						});
					} else {
						video.parents('.post_featured').removeClass( 'with_video_sticky_on' );
						video.height('auto').removeClass('trx_addons_video_sticky trx_addons_video_sticky_on');
						$video_sticky = jQuery('.trx_addons_video_sticky');
						$window.trigger('resize');
					}

					return false;
				} );
		});
	
	
		// Popups & Panels
		//----------------------------------------------

		// PrettyPhoto Engine
		if (TRX_ADDONS_STORAGE['popup_engine'] == 'pretty') {
			// Display lightbox on click on the image
			container
				.find('a[href$="jpg"]:not(.inited):not([target="_blank"])'
					+',a[href$="jpeg"]:not(.inited):not([target="_blank"])'
					+',a[href$="png"]:not(.inited):not([target="_blank"])'
					+',a[href$="gif"]:not(.inited):not([target="_blank"])')
				.each( function() {
					var $self = jQuery(this);
					if ( ! $self.parent().hasClass('woocommerce-product-gallery__image')) {
						$self.attr('rel', 'prettyPhoto[slideshow]');
					}
				});
			var images = container.find('a[rel*="prettyPhoto"]:not(.inited)'
										+ ':not(.esgbox)'
										+ ':not(.fancybox)'
										+ ':not([target="_blank"])'
										+ ':not([data-rel*="pretty"])'
										+ ':not([rel*="magnific"])'
										+ ':not([data-rel*="magnific"])'
										+ ':not([data-elementor-open-lightbox="yes"])'
										+ ':not([data-elementor-open-lightbox="default"])'
										).addClass('inited');
			try {
				images.prettyPhoto({
					social_tools: '',
					theme: 'facebook',
					deeplinking: false
				});
			} catch (e) {};
		
		// or Magnific Popup Engine
		} else if (TRX_ADDONS_STORAGE['popup_engine']=='magnific' && typeof jQuery.fn.magnificPopup != 'undefined') {
			// Display lightbox on click on the image
			container
				.find('a[href$="jpg"]:not(.inited):not([target="_blank"])'
					+',a[href$="jpeg"]:not(.inited):not([target="_blank"])'
					+',a[href$="png"]:not(.inited):not([target="_blank"])'
					+',a[href$="gif"]:not(.inited):not([target="_blank"])')
				.each( function() {
					var $self = jQuery(this);
					if ( $self.parents('.cq-dagallery').length == 0
						&& ! $self.hasClass('prettyphoto')
						&& ! $self.hasClass('esgbox')
					) {
						$self.attr('rel', 'magnific');
					}
				});
			var images = container.find('a[rel*="magnific"]:not(.inited)'
										+ ':not(.esgbox)'
										+ ':not(.fancybox)'
										+ ':not([target="_blank"])'
										+ ':not(.prettyphoto)'
										+ ':not([rel*="pretty"])'
										+ ':not([data-rel*="pretty"])'
										+ ':not([data-elementor-open-lightbox="yes"])'
										+ ':not([data-elementor-open-lightbox="default"])'
										).addClass('inited');
			// Unbind prettyPhoto
			setTimeout(function() {	images.unbind('click.prettyphoto'); }, 100);
			// Bind Magnific
			try {
				images.magnificPopup({
					type: 'image',
					mainClass: 'mfp-img-mobile',
					closeOnContentClick: true,
					closeBtnInside: true,
					fixedContentPos: true,
					midClick: true,
					//removalDelay: 500, 
					preloader: true,
					tLoading: TRX_ADDONS_STORAGE['msg_magnific_loading'],
					tClose: TRX_ADDONS_STORAGE['msg_magnific_close'],
					closeMarkup: '<button title="%title%" type="button" class="mfp-close"><span class="mfp-close-icon">&#215;</span></button>',
					gallery:{
						enabled: true
					},
					image: {
						tError: TRX_ADDONS_STORAGE['msg_magnific_error'],
						verticalFit: true,
						titleSrc: function(item) {
							var title = '',
								$el = typeof item.el != 'undefined' ? jQuery( item.el ) : null;
							if ( $el && $el.length > 0 ) {
								var $next = $el.next();
								if ( $next.length > 0 && $next.get(0).tagName == 'FIGCAPTION' ) {
									title = $next.text();
								} else if ( $el.attr( 'title' ) ) {
									title = $el.attr( 'title' );
								} else {
									var $img = $el.find( 'img' );
									if ( $img.length > 0 ) {
										title = $img.attr( 'alt' );
									}
								}
							}
							return title;
						}
					},
					zoom: {
						enabled: true,
						duration: 300,
						easing: 'ease-in-out',
						opener: function(openerElement) {
							// openerElement is the element on which popup was initialized, in this case its <a> tag
							// you don't need to add "opener" option if this code matches your needs, it's defailt one.
							if (!openerElement.is('img')) {
								if (openerElement.parents('.trx_addons_hover').find('img').length > 0)
									openerElement = openerElement.parents('.trx_addons_hover').find('img');
								else if (openerElement.find('img').length > 0)
									 openerElement = openerElement.find('img');
								else if (openerElement.siblings('img').length > 0)
									 openerElement = openerElement.siblings('img');
								else if (openerElement.parent().parent().find('img').length > 0)
									 openerElement = openerElement.parent().parent().find('img');
							}
							return openerElement; 
						}
					},
					callbacks: {
						beforeClose: function(){
							jQuery('.mfp-figure figcaption').hide();
							jQuery('.mfp-figure .mfp-arrow').hide();
						}
					}
				});
			} catch (e) {};


			// Prepare links to popups & panels
			var on_leaving_site = [],
				in_page_edit_mode = $body.hasClass('elementor-editor-active')
									|| $body.hasClass('wp-admin')
									|| $body.hasClass('block-editor-page');
			container.find('.sc_layouts_popup:not(.inited),.sc_layouts_panel:not(.inited)').each( function() {
				var $self = jQuery(this),
					id = $self.attr('id'),
					show = false;
				if (!id) return;
				var is_panel = $self.hasClass('sc_layouts_panel'),
					link = jQuery('a[href="#'+id+'"]');
				if (link.length == 0) {
					$body.append('<a href="#'+id+'" class="trx_addons_hidden"></a>');
					link = jQuery('a[href="#'+id+'"]');
				}
				if ($self.hasClass('sc_layouts_show_on_page_load')) {
					show = true;
				} else if ($self.hasClass('sc_layouts_show_on_page_load_once') && trx_addons_get_cookie('trx_addons_show_on_page_load_once_'+id) != '1') {
					trx_addons_set_cookie('trx_addons_show_on_page_load_once_'+id, '1');
					show = true;
				} else if ($self.hasClass('sc_layouts_show_on_page_close') && trx_addons_get_cookie('trx_addons_show_on_page_close_'+id) != '1') {
					on_leaving_site.push({
						link: link,
						id: id
					});
				}
				if (show) {
					// Display popups (panels) on the page (site) load
					if ( ! in_page_edit_mode ) {
						setTimeout( function() {
							link.trigger('click');
						}, $self.data('delay') > 0 ? $self.data('delay') * 1000 : 0 );
					}
				}
				link.addClass(is_panel ? 'trx_addons_panel_link' : 'trx_addons_popup_link')
					.data('panel', $self);
				$self.addClass('inited')
					.on('click', '.sc_layouts_panel_close', function(e) {
						trx_addons_close_panel($self);
						e.preventDefault();
						return false;
					});
			});
			
			// Display popup when user leaving site
			if ( on_leaving_site.length > 0 && ! in_page_edit_mode ) {
				var showed = false;
				$window.on( 'mousemove', function(e) {
					if ( showed ) return;
					var y = typeof e.clientY != 'undefined' ? e.clientY : 999;
					if ( y < _adminbar_height + 15 ) {
						showed = true;
						on_leaving_site.forEach( function(item) {
							item.link.trigger('click');
							trx_addons_set_cookie('trx_addons_show_on_page_close_'+item.id, '1');
						});
					}
				} );
			}

			// Close panel on click on the modal cover
			container.find('.sc_layouts_panel_hide_content:not(.inited)')
				.addClass('inited')
				.on('click', function(e) {
					trx_addons_close_panel(jQuery(this).next());
					e.preventDefault();
					return false;
				});


			// Display lightbox on click on the popup link
			container.find(".trx_addons_popup_link:not(.popup_inited)")
				.addClass('popup_inited')
				.magnificPopup({
					type: 'inline',
					focus: 'input',
					tLoading: TRX_ADDONS_STORAGE['msg_magnific_loading'],
					tClose: TRX_ADDONS_STORAGE['msg_magnific_close'],
					closeBtnInside: true,
					closeMarkup: '<button title="%title%" type="button" class="mfp-close"><span class="mfp-close-icon">&#215;</span></button>',
					callbacks: {
						// Will fire when this exact popup is opened
						// this - is Magnific Popup object
						open: function () {
							// Get saved content or store it (if first open occured)
							trx_addons_prepare_popup_content(this.content, true);
						},
						close: function () {
							// Save and remove content before closing
							// if its contain video, audio or iframe
							trx_addons_close_panel(this.content);
						},
						// resize event triggers only when height is changed or layout forced
						resize: function () {
							trx_addons_resize_actions(jQuery(this.content));
						}
					}
				});
	
			// Display panel on click on the panel link
			container.find(".trx_addons_panel_link:not(.panel_inited)")
				.addClass('panel_inited')
				.on('click', function(e) {
					var panel = jQuery(this).data('panel');
					if ( ! panel.hasClass( 'sc_layouts_panel_opened' ) ) {
						var body = $body;
						trx_addons_prepare_popup_content(panel, true);
						panel.addClass('sc_layouts_panel_opened');
						if (panel.prev().hasClass('sc_layouts_panel_hide_content')) panel.prev().addClass('sc_layouts_panel_opened');
						body.addClass('sc_layouts_panel_opened sc_layouts_panel_opened_' + panel.data('panel-position'));
						var panel_class = panel.data('panel-class');
						if ( panel_class ) {
							body.addClass( panel_class + '_opened' );
						}
					} else {
						trx_addons_close_panel(panel);
					}
					e.preventDefault();
					return false;
				});
			
			// Close panel
			window.trx_addons_close_panel = function(panel) {
				var body = $body;
				panel.removeClass('sc_layouts_panel_opened');
				if (panel.prev().hasClass('sc_layouts_panel_hide_content')) panel.prev().removeClass('sc_layouts_panel_opened');
				body.removeClass('sc_layouts_panel_opened sc_layouts_panel_opened_left sc_layouts_panel_opened_right sc_layouts_panel_opened_top sc_layouts_panel_opened_bottom');
				var panel_class = panel.data('panel-class');
				if ( panel_class ) {
					body.removeClass( panel_class + '_opened' );
				}
				if (panel.data('popup-content') !== undefined) {
					setTimeout(function() { panel.empty(); }, 500);
				}
			};

			// Get saved content for panel or popup or store it (if first open occured)
			window.trx_addons_prepare_popup_content = function(container, autoplay) {
				var wrapper = jQuery(container);
				// Store popup content to the data-param or restore it when popup open again (second time)
				// if popup contains audio or video or iframe
				if (wrapper.data('popup-content') === undefined) {
					var html = wrapper.html();
					if (html.search(/\<(audio|video|iframe)/i) >= 0) {
						wrapper.data('popup-content', html);
					}
				} else {
					wrapper.html(wrapper.data('popup-content'));
					// Remove class 'inited' to reinit elements
					wrapper.find('.inited').removeClass('inited');
				}
				// Replace src with data-src
				wrapper.find('[data-src]').each(function() {
					jQuery(this).attr( 'src', jQuery(this).data('src') );
				});
				// Init hidden elements
				$document.trigger('action.init_hidden_elements', [wrapper]);
				// Init third-party plugins in the popup
				$document.trigger('action.init_popup_elements', [wrapper]);
				// If popup contain embedded video - add autoplay
				if (autoplay) trx_addons_set_autoplay(wrapper);
				// If popup contain essential grid
				var frame = wrapper.find('.esg-grid');
				if (frame.length > 0) {
					var wrappers = [".esg-tc.eec", ".esg-lc.eec", ".esg-rc.eec", ".esg-cc.eec", ".esg-bc.eec"];
					for (var i=0; i<wrappers.length; i++) {
						frame.find(wrappers[i]+'>'+wrappers[i]).unwrap();
					}
				}
				// Call resize actions for the new content
				//trx_addons_resize_actions(wrapper);
				$window.trigger('resize');
			};
		}
	
	
		// Likes counter
		//---------------------------------------------
		var $likes = container.find('a.post_meta_likes:not(.inited),a.comment_counters_likes:not(.inited)');
		if ($likes.length > 0) {
			var likes_busy = false;
			$likes
				.addClass('inited')
				.on('click', function(e) {
					if ( ! likes_busy) {
						likes_busy = true;
						var button = jQuery(this);
						var inc = button.hasClass('enabled') ? 1 : -1;
						var post_id = button.hasClass('post_meta_likes') ? button.data('postid') :  button.data('commentid');
						var cookie_likes = trx_addons_get_cookie(button.hasClass('post_meta_likes') ? 'trx_addons_likes' : 'trx_addons_comment_likes');
						if (cookie_likes === undefined || cookie_likes===null) cookie_likes = '';
						jQuery.post( TRX_ADDONS_STORAGE['ajax_url'], {
							action: button.hasClass('post_meta_likes') ? 'post_counter' : 'comment_counter',
							nonce: TRX_ADDONS_STORAGE['ajax_nonce'],
							post_id: post_id,
							likes: inc
						}).done(function(response) {
							var rez = {};
							try {
								rez = JSON.parse(response);
							} catch (e) {
								rez = { error: TRX_ADDONS_STORAGE['msg_ajax_error'] };
								console.log(response);
							}
							if (rez.error === '') {
								var counter = rez.counter;
								if (inc == 1) {
									var title = button.data('title-dislike');
									button.removeClass('enabled trx_addons_icon-heart-empty').addClass('disabled trx_addons_icon-heart');
									cookie_likes += (cookie_likes.substr(-1)!=',' ? ',' : '') + post_id + ',';
								} else {
									var title = button.data('title-like');
									button.removeClass('disabled trx_addons_icon-heart').addClass('enabled trx_addons_icon-heart-empty');
									cookie_likes = cookie_likes.replace(','+post_id+',', ',');
								}
								button.data('likes', counter).attr('title', title).find(button.hasClass('post_meta_likes') ? '.post_meta_number' : '.comment_counters_number').html(counter);
								trx_addons_set_cookie( button.hasClass('post_meta_likes') ? 'trx_addons_likes' : 'trx_addons_comment_likes', cookie_likes, 365 * 24 * 60 * 60 * 1000);
							} else {
								alert(TRX_ADDONS_STORAGE['msg_error_like']);
							}
							likes_busy = false;
							$document.trigger('action.got_ajax_response', {
								action: button.hasClass('post_meta_likes') ? 'post_counter' : 'comment_counter',
								result: rez
							});
						});
					}
					e.preventDefault();
					return false;
				});
		}
	
	
		// Emotions counter
		//---------------------------------------------
		var $emotions = container.find('.trx_addons_emotions:not(.inited)');
		if ($emotions.length > 0) {
			var emotions_busy = false;
			$emotions
				.addClass('inited')
				.on('click', '.trx_addons_emotions_item', function(e) {
					if (!emotions_busy) {
						emotions_busy = true;
						var button = jQuery(this);
						var button_active = button.parent().find('.trx_addons_emotions_active');
						var post_id = button.data('postid');
						jQuery.post(TRX_ADDONS_STORAGE['ajax_url'], {
							action: 'post_counter',
							nonce: TRX_ADDONS_STORAGE['ajax_nonce'],
							post_id: post_id,
							emotion_inc: button.data('slug'),
							emotion_dec: button_active.length > 0 ? button_active.data('slug') : '',
						}).done(function(response) {
							var rez = {};
							try {
								rez = JSON.parse(response);
							} catch (e) {
								rez = { error: TRX_ADDONS_STORAGE['msg_ajax_error'] };
								console.log(response);
							}
							if (rez.error === '') {
								var cookie_likes = trx_addons_get_cookie('trx_addons_emotions'),
									cookie_likes_new = ',';
								if (cookie_likes) {
									cookie_likes = cookie_likes.split(',');
									for (var i=0; i<cookie_likes.length; i++) {
										if (cookie_likes[i] === '') continue;
										var tmp = cookie_likes[i].split('=');
										if (tmp[0] != post_id) cookie_likes_new += cookie_likes[i] + ',';
									}
								}
								cookie_likes = cookie_likes_new;
								if (button_active.length > 0) {
									button_active.removeClass('trx_addons_emotions_active');
								}
								if (button_active.length == 0 || button.data('slug') != button_active.data('slug')) {
									button.addClass('trx_addons_emotions_active');
									cookie_likes += (cookie_likes.substr(-1)!=',' ? ',' : '') + post_id + '=' + button.data('slug') + ',';
								}
								for (var i in rez.counter) {
									button.parent().find('[data-slug="'+i+'"] .trx_addons_emotions_item_number').html(rez.counter[i]);
								}
								trx_addons_set_cookie('trx_addons_emotions', cookie_likes, 365 * 24 * 60 * 60 * 1000);
							} else {
								alert(TRX_ADDONS_STORAGE['msg_error_like']);
							}
							emotions_busy = false;
							$document.trigger('action.got_ajax_response', {
								action: 'post_counter',
								result: rez
							});
						});
					}
					e.preventDefault();
					return false;
				});
		}
	
	
		// Socials share
		//----------------------------------------------
		var $share_caption = container.find('.socials_share .socials_caption:not(.inited)');
		if ($share_caption.length > 0) {
			$share_caption.each(function() {
				jQuery(this).addClass('inited').on('click', function(e) {
					jQuery(this).siblings('.social_items').slideToggle();	//.toggleClass('opened');
					e.preventDefault();
					return false;
				});
			});
		}
		var $share_items = container.find('.socials_share .social_items:not(.inited)');
		if ($share_items.length > 0) {
			$share_items.each(function() {
				jQuery(this)
					.addClass('inited')
					.on('click', '.social_item_popup', function(e) {
						var url = jQuery(this).data('link');
						window.open(url, '_blank', 'scrollbars=0, resizable=1, menubar=0, left=100, top=100, width=480, height=400, toolbar=0, status=0');
						e.preventDefault();
						return false;
					})
					.on('click', '.social_item[data-copy-link-url]', function(e) {
						var $self = jQuery(this),
							url = $self.data('copy-link-url');
						if ( url != '' ) {
							trx_addons_copy_to_clipboard( url );
							var msg = $self.data('message') ? $self.data('message') : TRX_ADDONS_STORAGE['msg_copied'];
							if ( msg ) {
								$self.attr('data-tooltip-text', msg);
								setTimeout( function() {
									$self.removeAttr('data-tooltip-text');
								}, 3000 );
							}
						}
						e.preventDefault();
						return false;
					});
			});
		}

		
		// Banners
		//-----------------------------------------------
		$banner_placeholder.each( function() {
			var item = jQuery(this);
			if ( ! item.hasClass( 'inited' ) && item.data('banner-show') == 'permanent' ) {
				item.addClass( 'inited' );
				setTimeout( function() {
					item.after( item.data( 'banner' ) );
					var banner = item.next();
					item.remove();
					if ( banner.hasClass( 'banner_hidden' ) ) {
						trx_addons_when_images_loaded( banner, function() {
							banner.slideDown();
						} );
					}
				}, item.data('banner-delay') ? item.data('banner-delay') : 0 );
			}
		} );
		
		
		// Widgets decoration
		//----------------------------------------------
	
		// Decorate nested lists in widgets and side panels
		container.find('.widget ul > li').each(function() {
			var $self = jQuery(this);
			if ($self.find('ul').length > 0) {
				$self.addClass('has_children');
			}
		});
	
		// Archive widget decoration
		container.find('.widget_archive a:not(.inited)').each(function() {
			var $self = jQuery(this).addClass('inited');
			var val = $self.html().split(' ');
			if (val.length > 1) {
				val[val.length-1] = '<span>' + val[val.length-1] + '</span>';
				$self.html(val.join(' '))
			}
		});
	
	
		// Menu
		//----------------------------------------------
	
		// Prepare menus (if menu cache is used)
		jQuery('.sc_layouts_menu_nav:not(.inited_cache)').each(function() {
			var $self = jQuery(this).addClass('inited_cache');
			if ($self.find('.current-menu-item').length == 0 || $body.hasClass('blog_template')) {
				if (TRX_ADDONS_STORAGE['menu_cache'] === undefined) TRX_ADDONS_STORAGE['menu_cache'] = [];
				var id = $self.attr('id');
				if (id === undefined) {
					id = ('sc_layouts_menu_nav_' + Math.random()).replace('.', '');
					$self.attr('id', id);
				}
				TRX_ADDONS_STORAGE['menu_cache'].push('#'+id);
			}
		});
		if (TRX_ADDONS_STORAGE['menu_cache'] && TRX_ADDONS_STORAGE['menu_cache'].length > 0) {
			// Mark the current menu item and its parent items in the cached menus
			var href = window.location.href;
			if (href.substr(-1)=='/') href = href.substr(0, href.length-1);
			var href2 = href + '/';
			for (var i = 0; i < TRX_ADDONS_STORAGE['menu_cache'].length; i++) {
				var menu = jQuery(TRX_ADDONS_STORAGE['menu_cache'][i]+':not(.prepared)');
				if (menu.length==0) continue;
				menu.addClass('prepared');
				menu.find('li').removeClass('current-menu-ancestor current-menu-parent current-menu-item current_page_item');
				menu.find('a[href="'+href+'"],a[href="'+href2+'"]').each(function(idx) {
					var li = jQuery(this).parent();
					li.addClass('current-menu-item');
					if (li.hasClass('menu-item-object-page')) li.addClass('current_page_item');
					var cnt = 0;
					while ((li = li.parents('li')).length > 0) {
						cnt++;
						li.addClass('current-menu-ancestor'+(cnt==1 ? ' current-menu-parent' : ''));
					}
				});
			}
		}
	
	
		// Forms
		//------------------------------------
		
		// Remove 'error_field' class on any key pressed in the field
		jQuery("form:not([data-inited-keypress])")
			.attr('data-inited-keypress', 1)
			.on('keypress', '.error_field', function() {
				var $self = jQuery(this);
				if ($self.val() !== '') {
					$self.removeClass('error_field');
				}
			});
	
		// Comment form
		jQuery("form#commentform:not(.inited_validation)")
			.addClass('inited_validation')
			.submit(function(e) {
				var rez = trx_addons_comments_validate(jQuery(this));
				if (!rez) {
					e.preventDefault();
				}
				return rez;
			});
		
		// Comments form
		function trx_addons_comments_validate(form) {
			form.find('input').removeClass('error_field');
			var comments_args = {
				error_message_text: TRX_ADDONS_STORAGE['msg_validation_error'],		// Global error message text (if don't write in checked field)
				error_message_show: true,											// Display or not error message
				error_message_time: 4000,											// Error message display time
				error_message_class: 'trx_addons_message_box trx_addons_message_box_error',	// Class appended to error message block
				error_fields_class: 'error_field',									// Class appended to error fields
				exit_after_first_error: false,										// Cancel validation and exit after first error
				rules: [
					{
						field: 'comment',
						min_length: { value: 1, message: TRX_ADDONS_STORAGE['msg_text_empty'] }
					}
				]
			};
			if (form.find('.comments_author input[aria-required="true"]').length > 0) {
				comments_args.rules.push(
					{
						field: 'author',
						min_length: { value: 1, message: TRX_ADDONS_STORAGE['msg_name_empty']},
						max_length: { value: 60, message: TRX_ADDONS_STORAGE['msg_name_long']}
					}
				);
			}
			if (form.find('.comments_email input[aria-required="true"]').length > 0) {
				comments_args.rules.push(
					{
						field: 'email',
						min_length: { value: 1, message: TRX_ADDONS_STORAGE['msg_email_empty']},
						max_length: { value: 60, message: TRX_ADDONS_STORAGE['msg_email_long']},
						mask: { value: TRX_ADDONS_STORAGE['email_mask'], message: TRX_ADDONS_STORAGE['msg_email_not_valid']}
					}
				);
			}
			var error = trx_addons_form_validate(form, comments_args);
			return !error;
		}
		

		// Animations
		//------------------------------------

		// Disable animations on mobile
		if ( TRX_ADDONS_STORAGE['disable_animation_on_mobile'] && $body.hasClass('ua_mobile') ) {
			jQuery('[data-animation^="animated"]').removeAttr('data-animation');
			jQuery('[data-hover-animation^="animated"]').removeAttr('data-hover-animation');
			// Update global lists
			$animated_elements = jQuery('[data-animation^="animated"]:not(.animated)');
			$animated_hover    = jQuery('[data-hover-animation^="animated"]:not(.animated)');
		}

		// Animated hover
		container
			.find('[data-hover-animation^="animated"]')
			.closest('.elementor-column,.post_layout_custom')
			.each(function() {
				var $self = jQuery(this);
				if ($self.hasClass('hover-animation-inited')) return;
				$self.addClass('hover-animation-inited').hover(
					// Mouse in
					function(e) {
						jQuery(this).find('[data-hover-animation^="animated"]').each(function() {
							var obj = jQuery(this);
							var animation = obj.data('hover-animation');
							var animation_in = obj.data('animation-in');
							if (animation_in == undefined) animation_in = "none";
							var animation_in_delay = obj.data('animation-in-delay');
							if (animation_in_delay == undefined) animation_in_delay = 0;
							var animation_out = obj.data('animation-out');
							if (animation_out == undefined) animation_out = "none";
							if (animation_in != 'none') {
								setTimeout(function() {
									obj.removeClass(animation + ' ' + animation_out);
									obj.addClass(animation + ' ' + animation_in);
								}, animation_in_delay);
							}
						});
					},
					// Mouse out
					function(e) {
						jQuery(this).find('[data-hover-animation^="animated"]').each(function() {
							var obj = jQuery(this);
							var animation = obj.data('hover-animation');
							var animation_in = obj.data('animation-in');
							if (animation_in == undefined) animation_in = "none";
							var animation_out = obj.data('animation-out');
							if (animation_out == undefined) animation_out = "none";
							var animation_out_delay = obj.data('animation-out-delay');
							if (animation_out_delay == undefined) animation_out_delay = 0;
							if (animation_out != 'none') {
								setTimeout(function() {
									obj.removeClass(animation + ' ' + animation_in);
									obj.addClass(animation + ' ' + animation_out);
								}, animation_out_delay);
							}
						});
					}
				);
			});

		// Init Parallax controller
		//-----------------------------------
		if ( typeof ScrollMagic != 'undefined' ) {
			if ( parallax_controller === null ) {
				parallax_controller = new ScrollMagic.Controller({globalSceneOptions: {triggerHook: "onEnter", duration: "200%"}});
			}
			// Build Parallax scenes
			$parallax_wrap.each( function() {
				var $self = jQuery( this ),
					id = $self.attr( 'id' ),
					speed = $self.data( 'parallax' );
				if ( speed != 0 && ! $self.hasClass( 'parallax_inited' ) ) {
					$self.addClass( 'parallax_inited' );
					if ( ! id ) {
						id = 'sc_parallax_wrap_' + ( '' + Math.random() ).replace( '.', '' );
						$self.attr( 'id', id );
					}
					var selector = '#' + id + ( $self.find( '> .wp-caption' ).length > 0 ? '>.wp-caption' : '' ) + '>img';
					new ScrollMagic
						.Scene( { triggerElement: '#' + id } )
						.setTween( selector, { y: speed + "%", ease: Linear.easeNone } )
						.addTo( parallax_controller );
				}
			});
		}


		// Other settings
		//------------------------------------

		// Scroll to top button
		if ( ! $scroll_to_top.hasClass('inited') ) {
			$scroll_to_top
				.addClass('inited')
				.on('click', function(e) {
					jQuery('html,body').animate( {
						scrollTop: 0
					}, 'slow' );
					e.preventDefault();
					return false;
				});
		}


		// Call plugins specific action (if exists)
		//----------------------------------------------
		$document.trigger('action.before_ready_trx_addons');
		$document.trigger('action.ready_trx_addons');
		$document.trigger('action.after_ready_trx_addons');
		
	} //end ready


	// Init mouse helper
	//--------------------------------
	function trx_addons_mouse_helper_init() {
		if ( TRX_ADDONS_STORAGE['mouse_helper'] > 0 && $mouse_helper.length > 0 && requestAnimationFrame ) {
			mouse_helper_color = $mouse_helper.css( 'color' );
			mouse_helper_bg_color = $mouse_helper.css( 'background-color' );
			mouse_helper_bd_color = $mouse_helper.css( 'border-color' );
			$document
				.on( 'mousemove', function(e) {
					if ( mouse_helper_callback && typeof window[mouse_helper_callback] == 'function' ) {
						window[mouse_helper_callback]( 'mousemove', mouse_helper_target, e );
					}
					if ( mouse_helper_axis.indexOf('x') != -1 ) mouseX = e.clientX;
					if ( mouse_helper_axis.indexOf('y') != -1 ) mouseY = e.clientY;
					if ( mouse_helper_action == 'highlight' && mouse_helper_target ) {
						var targetOffset = mouse_helper_target.offset(),
							targetX = targetOffset.left - _window_scroll_left,
							targetY = targetOffset.top - _window_scroll_top,
							size = parseFloat(mouse_helper_target.css('background-size').split(' ')[0])/2;
						mouse_helper_target.css('background-position', (mouseX-targetX-size)+'px '+(mouseY-targetY-size)+'px');
					}
					if ( _window_width >= TRX_ADDONS_STORAGE['mobile_breakpoint_mousehelper_off'] ) {
						if ( mouse_helper_delay < 2 ) {
							$mouse_helper.css("transform", "translate(" + mouseX + "px," + mouseY + "px)");
						}
						if ( mouseX > _window_width - 100 ) {
							if ( ! $mouse_helper.hasClass( 'trx_addons_mouse_helper_left' ) ) {
								$mouse_helper.addClass( 'trx_addons_mouse_helper_left' );
							}
						} else {
							if ( $mouse_helper.hasClass( 'trx_addons_mouse_helper_left' ) ) {
								$mouse_helper.removeClass( 'trx_addons_mouse_helper_left' );
							}
						}
						if ( mouseY > _window_height - 100 ) {
							if ( ! $mouse_helper.hasClass( 'trx_addons_mouse_helper_top' ) ) {
								$mouse_helper.addClass( 'trx_addons_mouse_helper_top' );
							}
						} else {
							if ( $mouse_helper.hasClass( 'trx_addons_mouse_helper_top' ) ) {
								$mouse_helper.removeClass( 'trx_addons_mouse_helper_top' );
							}
						}
					}
				} )
				.on("mouseenter", "[data-mouse-helper]", function(e) {
					var $self = jQuery(this);
					mouse_helper_target = $self;
					mouse_helper_action = $self.data( 'mouse-helper' );
					mouse_helper_callback = $self.data( 'mouse-helper-callback' );
					if ( mouse_helper_callback === undefined ) {
						mouse_helper_callback = '';
					} else if ( typeof window[mouse_helper_callback] == 'function' ) {
						window[mouse_helper_callback]( 'mouseenter', $self, e );
					}
					mouse_helper_axis = $self.data( 'mouse-helper-axis' );
					if ( mouse_helper_axis === undefined ) {
						mouse_helper_axis = 'xy';
					}
					mouse_helper_delay = $self.data( 'mouse-helper-delay' );
					if ( mouse_helper_delay === undefined ) {
						mouse_helper_delay = mouse_helper_delay_default;
					}
					if ( _window_width >= TRX_ADDONS_STORAGE['mobile_breakpoint_mousehelper_off'] ) {
						mouse_helper_reset();
						if ( mouse_helper_timeout ) {
							clearTimeout( mouse_helper_timeout );
							mouse_helper_timeout = 0;
						}
						var mode = $self.data("mouse-helper-mode") || '';
						if ( mode ) {
							$mouse_helper.css( 'mix-blend-mode', mode );
						}
						var color = $self.data("mouse-helper-color") || '';
						if ( color ) {
							$mouse_helper.css( 'color', color );
						}
						var bg_color = $self.data("mouse-helper-bg-color") || '';
						if ( bg_color ) {
							$mouse_helper.css( 'background-color', bg_color );
						}
						var bd_color = $self.data("mouse-helper-bd-color") || '';
						if ( bd_color ) {
							$mouse_helper.css( 'border-color', bd_color );
						}
						var pos = $self.data("mouse-helper-centered") || '';
						if ( pos ) {
							$mouse_helper
								.addClass( 'trx_addons_mouse_helper_centered' );
						}
						var icon = $self.data("mouse-helper-icon") || '';
						if ( icon ) {
							var icon_color = $self.data("mouse-helper-icon-color") || '';
							$mouse_helper
								.append( '<span class="trx_addons_mouse_helper_icon ' + icon + '"'
												+ ( icon_color ? ' style="color: ' + icon_color + ';"' : '' )
											+'></span>' )
								.addClass( 'trx_addons_mouse_helper_with_icon' );
						}
						var text = $self.data("mouse-helper-text") || '',
							text_round = $self.data("mouse-helper-text-round") || false;
						if ( text ) {
							$mouse_helper
								.append( '<span class="trx_addons_mouse_helper_text' + ( text_round ? ' trx_addons_mouse_helper_text_round' : '' ) + '">'
											+ ( text_round ? trx_addons_wrap_chars( text, '<span class="trx_addons_mouse_helper_text_round_item">', '</span>' ) : text )
											+ '</span>' )
								.addClass( 'trx_addons_mouse_helper_with_text' );
							if ( text_round ) {
								setTimeout( function() {
									var text = $mouse_helper.find( '.trx_addons_mouse_helper_text' ),
										items = text.find( '.trx_addons_mouse_helper_text_round_item' );
									if ( items.length > 0) {
										var b = parseFloat( $mouse_helper.css('border-left-width') ) || 0,
											r = ( $mouse_helper.outerWidth() - 2 * b ) / 2,
											a = 0,
											hide = false;
										items.each( function( idx ) {
											if ( hide ) {
												items.eq(idx).hide();
											} else {
												if ( a >= 340 ) {	// If fill whole round - hide rest chars and replace its with '...'
													items.eq(idx).html('&hellip;');
													hide = true;
												}
												var x = r + r * Math.sin( a / 180 * Math.PI ),
													y = r - r * Math.cos( a / 180 * Math.PI );
												items.eq(idx).css( {
													'transform': 'rotate(' + a + 'deg)',
													'left': x + 'px',
													'top':  y + 'px'
												} );
												a += 2 * Math.asin( ( items.eq(idx).width() + 3 ) / ( 2 * r ) ) * 180 / Math.PI;
											}
										} );
									}
								}, 350);
							}
						}
						var img = $self.data("mouse-helper-image") || '';
						if ( img ) {
							$mouse_helper
								.append( '<span class="trx_addons_mouse_helper_image" style="background-image:url(' + img + ');"></span>' )
								.addClass( "trx_addons_mouse_helper_with_image");
						}
						if ( icon || text || img ) {
							$mouse_helper.addClass( "trx_addons_mouse_helper_active");
						}
					}
				} )
				.on("mouseleave", "[data-mouse-helper]", function(e) {
					if ( mouse_helper_callback && typeof window[mouse_helper_callback] == 'function' ) {
						window[mouse_helper_callback]( 'mouseleave', mouse_helper_target, e );
						mouse_helper_callback = '';
					}
					mouse_helper_target = null;
					mouse_helper_action = '';
					mouse_helper_axis = 'xy';
					mouse_helper_delay = mouse_helper_delay_default;
					if ( _window_width >= TRX_ADDONS_STORAGE['mobile_breakpoint_mousehelper_off'] ) {
						mouse_helper_timeout = setTimeout( mouse_helper_reset, 300 );
					}
				} )
				.trigger( 'mousemove' );

			var mouse_helper_reset = function() {
				$mouse_helper
					.css( {
						'mix-blend-mode': 'initial',
						'color': mouse_helper_color,
						'background-color': mouse_helper_bg_color,
						'border-color': mouse_helper_bd_color
					} )
					.empty()
					.removeClass(
						"trx_addons_mouse_helper_active"
						+ " trx_addons_mouse_helper_with_icon"
						+ " trx_addons_mouse_helper_with_text"
						+ " trx_addons_mouse_helper_text_round"
						+ " trx_addons_mouse_helper_with_image"
						+ " trx_addons_mouse_helper_centered"
						);
			};

			var mouse_helper_move = function() {
				cancelAnimationFrame(mouse_helper_move);
				if ( _window_width >= TRX_ADDONS_STORAGE['mobile_breakpoint_mousehelper_off'] && null != mouseX && ( destX != mouseX || destY != mouseY ) && mouse_helper_delay > 1 ) {
					if ( $mouse_helper.hasClass( 'trx_addons_mouse_helper_permanent' ) || $mouse_helper.hasClass( 'trx_addons_mouse_helper_active' ) ) {
						destX += (mouseX - destX) / mouse_helper_delay;
						destY += (mouseY - destY) / mouse_helper_delay;
					} else {
						destX = mouseX;
						destY = mouseY;
					}
					$mouse_helper.css("transform", "translate(" + destX + "px," + destY + "px)");
				}
				requestAnimationFrame(mouse_helper_move);
			};
			requestAnimationFrame(mouse_helper_move);
		}
	}


	// Init bg canvas
	//--------------------------------
	function trx_addons_bg_canvas_init() {
		var cnt = 0;

		jQuery( '[data-bg-canvas-type="start"]' ).each( function() {
			var $start = jQuery( this ),
				id = $start.data('bg-canvas-id'),
				$end = $body.find( '[data-bg-canvas-id="' + id + '"][data-bg-canvas-type="end"]' ),
				effect_start = $start.data('bg-canvas-effect') ? $start.data('bg-canvas-effect') : 'round',
				effect_end = $end.data('bg-canvas-effect') ? $end.data('bg-canvas-effect') : 'round',
				color_start = $start.data('bg-canvas-color'),
				color_end = $end.data('bg-canvas-color') ? $end.data('bg-canvas-color') : color_start;

			if ( id && color_start && $end.length > 0 ) {
				if ( _bg_canvas === false ) {
					_bg_canvas = [];
				}

				$body
					.addClass( 'with_bg_canvas' )
					.append( '<div id="bg_canvas_' + id + '" class="bg_canvas"></div>' );

				_bg_canvas.push( {
					canvas: {
							obj: $body.find( '#bg_canvas_' + id ),
							},
					start: {
							obj: $start,
							color: color_start,
							effect: effect_start,
							size: $start.data('bg-canvas-size') ? $start.data('bg-canvas-size') : 0
							},
					end: {
							obj: $end,
							color: color_end,
							effect: effect_end,
							size: $end.data('bg-canvas-size') ? $end.data('bg-canvas-size') : 0
						}
				} );

				if ( effect_start == 'round' || effect_end == 'round' ) {
					_bg_canvas[cnt].canvas.cnv = document.createElement("canvas");
					_bg_canvas[cnt].canvas.ctx = _bg_canvas[cnt].canvas.cnv.getContext("2d");
					_bg_canvas[cnt].canvas.obj.append( _bg_canvas[cnt].canvas.cnv );
				}

				if ( effect_start == 'fade' || effect_end == 'fade' ) {
					_bg_canvas[cnt].canvas.obj.css( {
						'background-color': color_start,
						'opacity': 0
					} );
				}

				cnt++;
			}
		} );

		if ( _bg_canvas !== false ) {
			$document.on( 'action.resize_trx_addons', trx_addons_bg_canvas_set_positions );
			$document.on( 'action.got_ajax_response', function() {
				trx_addons_bg_canvas_set_positions();
				setTimeout( function() {
					trx_addons_bg_canvas_set_positions();
				}, 1000 );
			} );
			$window.on( 'action.scroll_trx_addons', function() {
				trx_addons_bg_canvas_draw( false );
			} );
			trx_addons_bg_canvas_set_positions();

			$document.trigger( 'action.trx_addons_bg_canvas', [_bg_canvas] );
		}

		function trx_addons_bg_canvas_draw( force ) {
			for (var i=0; i < _bg_canvas.length; i++ ) {
				trx_addons_bg_canvas_draw_item( _bg_canvas[i], force );
			}
		}

		function trx_addons_bg_canvas_set_positions() {
			for (var i=0; i < _bg_canvas.length; i++ ) {
				trx_addons_bg_canvas_item_set_position( _bg_canvas[i] );
			}
			trx_addons_bg_canvas_draw( true );
		}

		function trx_addons_bg_canvas_item_set_position( item ) {
			var o;

			jQuery( item.canvas.cnv )
				.css( {
					width: _window_width + 'px',
					height: _window_height + 'px'
				} )
				.attr( {
					width: _window_width,
					height: _window_height
				} );

			item.w = _window_width;
			item.h = _window_height;
			item.size = Math.sqrt(Math.pow(_window_width, 2) + Math.pow(_window_height, 2));

			o = item.start.obj.offset();
			item.start.w = Math.max(1, item.start.obj.width());
			item.start.h = Math.max(1, item.start.obj.height());
			item.start.left = o.left;
			item.start.top  = o.top;

			o = item.end.obj.offset();
			item.end.w = Math.max(1, item.end.obj.width());
			item.end.h = Math.max(1, item.end.obj.height());
			item.end.left = o.left;
			item.end.top  = o.top;
		}

		function trx_addons_bg_canvas_set_coords( item ) {
			
			item.cur_scroll = _window_scroll_top;

			var t  = item.start.top - item.cur_scroll,
				rt = 1 - Math.max( 0, Math.min( 1, t / _window_height ) ),
				b  = item.end.top + item.end.h - _window_height - item.cur_scroll,
				rb = Math.max( 0, Math.min( 1, b / _window_height ) );
			item.progress = Math.min(rt, rb);
			item.distance = rt == 1 && rb == 1 ? -t / ( -t + b - _window_height ) : 0;
			item.coords = item.end.top - _window_scroll_top < _window_height * 1.5 || rb < 1
							? {
								x: item.end.left + item.end.w / 2,
								y: item.end.top + item.end.h / 2 - _window_scroll_top,
								r: item.end.size,
								d: 'end'
								}
							: {
								x: item.start.left + item.start.w / 2,
								y: Math.max(item.start.top + item.start.h / 2 - _window_scroll_top, _window_height / 2),
								r: item.start.size,
								d: 'start'
								};
		}

		function trx_addons_bg_canvas_draw_item( item, force ) {
			trx_addons_bg_canvas_set_coords( item );
			if ( force || ! item.done || ( item.progress > 0 && item.progress < 1 ) ) {
				var effect = item.coords.d == 'start' ? item.start.effect : item.end.effect,
					color = item.coords.d == 'start' ? item.start.color : item.end.color;
				if ( item.start.color != item.end.color && item.distance > 0 ) {
					var rgb_start = trx_addons_hex2rgb( item.start.color ),
						rgb_end = trx_addons_hex2rgb( item.end.color );
					rgb_start['r'] += Math.round( ( rgb_end['r'] - rgb_start['r'] ) * item.distance );
					rgb_start['g'] += Math.round( ( rgb_end['g'] - rgb_start['g'] ) * item.distance );
					rgb_start['b'] += Math.round( ( rgb_end['b'] - rgb_start['b'] ) * item.distance );
					color = trx_addons_components2hex(rgb_start['r'],rgb_start['g'],rgb_start['b']);
				}
				if ( effect == 'round' ) {
					if ( item.effect != effect ) {
						item.effect = effect;
						item.canvas.obj.css( {
							'background-color': 'transparent',
							'opacity': 1
						} );
						jQuery( item.canvas.cnv ).show();
					}
					var t = item.progress * item.size;
					t = t < item.coords.r + 10 ? item.coords.r : t;
					item.canvas.ctx.clearRect(0, 0, item.w, item.h);
					item.canvas.ctx.beginPath();
					item.canvas.ctx.fillStyle = color;
					item.canvas.ctx.ellipse(item.coords.x, item.coords.y, t, t, 0, 0, 2 * Math.PI);
					item.canvas.ctx.closePath();
					item.canvas.ctx.fill();
				} else if ( effect == 'fade' ) {
					if ( item.effect != effect || item.start.color != item.end.color ) {
						item.canvas.obj.css( {
							'background-color': color
						} );
					}
					if ( item.effect != effect ) {
						item.effect = effect;
						jQuery( item.canvas.cnv ).hide();
					}
					item.canvas.obj.css( {
						'opacity': item.progress
					} );
				} else {
					$document.trigger( 'action.trx_addons_bg_canvas_draw', [item] );
				}
			}
			item.done = ( item.progress == 0 || item.progress == 1 )
						&& item.start.color == item.end.color
						&& ( item.end.top + item.end.h / 2 + item.end.size < _window_scroll_top || item.end.top > _window_scroll_top + _window_height );
		}
	}


	// Increment post views counter via AJAX
	//--------------------------------------
	if (TRX_ADDONS_STORAGE['ajax_views']) {
		$document.on('action.ready_trx_addons', function() {
			if (!TRX_ADDONS_STORAGE['post_views_counter_inited']) {
				TRX_ADDONS_STORAGE['post_views_counter_inited'] = true;
				setTimeout(function() {
					jQuery.post(TRX_ADDONS_STORAGE['ajax_url'], {
						action: 'post_counter',
						nonce: TRX_ADDONS_STORAGE['ajax_nonce'],
						post_id: TRX_ADDONS_STORAGE['post_id'],
						views: 1
					}).done(function(response) {
						var rez = {};
						try {
							rez = JSON.parse(response);
						} catch (e) {
							rez = { error: TRX_ADDONS_STORAGE['ajax_error'] };
							console.log(response);
						}
						if (rez.error === '') {
							jQuery('.post_meta_single .post_meta_views .post_meta_number,.sc_layouts_title_meta .post_meta_views .post_meta_number').html(rez.counter);
						}
						$document.trigger('action.got_ajax_response', {
							action: 'post_counter',
							result: rez
						});
					});
				}, 10);
			}
		});
	}

	
	// Scroll actions
	//==============================================

	// Do actions when page scrolled
	window.trx_addons_scroll_actions = function() {

		// Update global values
		_window_scroll_top  = $window.scrollTop();
		_window_scroll_left = $window.scrollLeft();

		// Scroll to top button show/hide
		if ($scroll_to_top.length > 0) {
			if (_window_scroll_top > 100)
				$scroll_to_top.addClass('show');
			else
				$scroll_to_top.removeClass('show');
		}

		// Show on scroll blocks
		$show_on_scroll.each( function() {
			var item = jQuery(this);
			if ( item.offset().top < _window_scroll_top + _window_height * 0.75 ) {
				item.removeClass( 'trx_addons_show_on_scroll' ).addClass( 'trx_addons_showed_on_scroll' );
				// Update list of elements
				$show_on_scroll = jQuery('.trx_addons_show_on_scroll');
			}
		} );

		// Show on scroll banners
		$banner_placeholder.each( function() {
			var item = jQuery(this);
			if ( ! item.hasClass('inited') && item.data('banner-show') == 'scroll' && item.offset().top < _window_scroll_top + _window_height * 1.5 ) {
				item.addClass('inited');
				setTimeout( function() {
					item.after( item.data( 'banner' ) );
					var banner = item.next();
					item.remove();
					if ( banner.hasClass( 'banner_hidden' ) ) {
						trx_addons_when_images_loaded( banner, function() {
							banner.slideDown();
						} );
					}
				}, item.data('banner-delay') ? item.data('banner-delay') : 0 );
			}
		} );

		// Scroll actions for animated elements
		$animated_elements.each(function(idx) {
			var item = jQuery(this);
			if ( ! item.hasClass('inited_animation') && item.offset().top < _window_scroll_top + _window_height) {
				item.addClass('inited_animation');
				setTimeout( function() {
					item.addClass(item.data('animation'));
					// Update global list
					$animated_elements = jQuery('[data-animation^="animated"]:not(.animated)');
				}, 100 * trx_addons_random(0, 10) );
			}
		});

		// Display scroll progress
		if ( ['top', 'bottom', 'fixed'].indexOf( TRX_ADDONS_STORAGE['scroll_progress'] ) >= 0 ) {
			trx_addons_show_scroll_progress();
		}

		// Fix/unfix sticky videos
		if ( $video_sticky.length > 0 ) {
			var video = $video_sticky.eq(0),
				video_top = video.offset().top,
				video_height = video.height();
			if ( video_top + video_height < _window_scroll_top + _adminbar_height ) {
				if ( ! video.hasClass('trx_addons_video_sticky_on') ) {
					video.parents('.post_featured').addClass( 'with_video_sticky_on' );
					if ( _video_sticky_fade ) {
						video.height( video_height ).css('opacity', 0).addClass( 'trx_addons_video_sticky_on' ).stop().animate({opacity:1}, 500);
					} else {
						video.height( video_height ).addClass( 'trx_addons_video_sticky_on' );
					}
				}
			} else {
				if ( video.hasClass('trx_addons_video_sticky_on') && ! video.hasClass('trx_addons_video_sticky_on_fade') ) {
					if ( _video_sticky_fade ) {
						video.addClass('trx_addons_video_sticky_on_fade').stop().animate({opacity:0}, 300, function() {
							video.parents('.post_featured').removeClass( 'with_video_sticky_on' );
							video.height('auto').removeClass( 'trx_addons_video_sticky_on trx_addons_video_sticky_on_fade' ).stop().animate({'opacity': 1}, 500);
							$window.trigger('resize');
						});
					} else {
						video.parents('.post_featured').removeClass( 'with_video_sticky_on' );
						video.height('auto').removeClass( 'trx_addons_video_sticky_on' );
						$window.trigger('resize');
					}
				}
			}
		}

		// Call theme/plugins specific action (if exists)
		//----------------------------------------------
		$document.trigger('action.before_scroll_trx_addons');
		$document.trigger('action.scroll_trx_addons');
		$document.trigger('action.after_scroll_trx_addons');

		// Set flag about scroll actions are finished
		TRX_ADDONS_STORAGE['scroll_busy'] = false;
	};


	// Display scroll progress
	function trx_addons_show_scroll_progress() {
		if ( TRX_ADDONS_STORAGE['scroll_progress_status'] == undefined ) {
			$body.append('<div class="scroll_progress_wrap scroll_progress_'+TRX_ADDONS_STORAGE['scroll_progress']+'"><span class="scroll_progress_status"></span></div>');
			TRX_ADDONS_STORAGE['scroll_progress_status'] = jQuery( '.scroll_progress_wrap .scroll_progress_status' );
			trx_addons_get_scroll_posts();
			// Animate to relative page part on click
			TRX_ADDONS_STORAGE['scroll_progress_status'].on('click', function(e) {
				var prc = e.pageX / jQuery(this).parent().width();
				if ( TRX_ADDONS_STORAGE['scroll_posts'] != undefined && TRX_ADDONS_STORAGE['scroll_posts'].length > 0 ) {
					var cur_post = trx_addons_detect_current_scroll_post(),
						pt = cur_post.data('post-top'),
						ph = cur_post.data('post-height');
					trx_addons_document_animate_to( Math.round( ph * prc + pt - wh / 2 ) );
				} else {
					trx_addons_document_animate_to( Math.round( ( _document_height - _window_height ) * prc ) );
				}
				e.preventDefault();
				return false;
			});
		}
		var st = _window_scroll_top,
			wh = _window_height,
			new_width = '0%';
		if ( TRX_ADDONS_STORAGE['scroll_posts'] !== undefined && TRX_ADDONS_STORAGE['scroll_posts'].length > 0 ) {
			var cur_post = trx_addons_detect_current_scroll_post(),
				pt = cur_post.data('post-top'),
				ph = cur_post.data('post-height');
			new_width = ( st < 10 ? 0 : Math.min( 100, Math.round( ( st + wh / 2 - pt ) * 100 / ph ) ) ) + '%';
			TRX_ADDONS_STORAGE['scroll_progress_status'].width( new_width );
		} else {
			new_width = Math.min( 100, Math.round( st * 100 / ( _document_height - wh ) ) ) + '%';
			TRX_ADDONS_STORAGE['scroll_progress_status'].width( new_width );
		}
	}

	function trx_addons_detect_current_scroll_post() {
		var cur_post = false;
		TRX_ADDONS_STORAGE['scroll_posts'].each( function() {
			var post = jQuery(this),
				st   = _window_scroll_top,
				wh   = _window_height,
				pt   = post.data('post-top'),
				ph   = post.data('post-height');
			if ( pt < st + wh / 2 ) {
				cur_post = post;
			}
		});
		if ( ! cur_post ) {
			cur_post = TRX_ADDONS_STORAGE['scroll_posts'].eq( TRX_ADDONS_STORAGE['scroll_posts'].length - 1 );
		}
		return cur_post;
	}

	$document.on('action.new_post_added', trx_addons_get_scroll_posts);
	function trx_addons_get_scroll_posts() {
		TRX_ADDONS_STORAGE['scroll_posts'] = ( TRX_ADDONS_STORAGE['scroll_posts'] !== undefined && TRX_ADDONS_STORAGE['scroll_posts'].length > 0 )
												|| jQuery('.nav-links-single-scroll').length > 0
													? jQuery('.post_item_single')
													: false;
		trx_addons_get_scroll_posts_dimensions();
	}

	$document.on('action.resize_trx_addons', trx_addons_get_scroll_posts_dimensions);
	function trx_addons_get_scroll_posts_dimensions() {
		if ( TRX_ADDONS_STORAGE['scroll_posts'] !== undefined && TRX_ADDONS_STORAGE['scroll_posts'].length > 0 ) {
			TRX_ADDONS_STORAGE['scroll_posts'].each( function() {
				var post = jQuery(this);
				post.data('post-height', post.height())
					.data('post-top', post.offset().top);
			} );
		}
	}


	// Fix columns
	//---------------------------------------------------------------

	// Fix column only if css sticky behaviour is not used
	if ( ! $body.hasClass( 'fixed_blocks_sticky' ) ) {
		$document.on('action.resize_trx_addons', trx_addons_fix_column);
		$document.on('action.scroll_trx_addons', trx_addons_fix_column);
		
		function trx_addons_fix_column(e, cont) {

			if ( $fixed_columns.length == 0 ) {
				return;
			}

			var force = e.namespace == 'resize_trx_addons';

			$fixed_columns.each(function() {
				var col = jQuery(this),
					row = col.parent();
				
				// Exit if non-standard responsive is used for this columns
				if ( col.attr('class').indexOf('vc_col-lg-') != -1 || col.attr('class').indexOf('vc_col-md-') != -1 ) {
					return;

				// Unfix on mobile layout (all columns are fullwidth)
				} else if ( _window_width < TRX_ADDONS_STORAGE['mobile_breakpoint_fixedcolumns_off'] ) {
					var old_style = col.data('old_style');
					if (old_style !== undefined) {
						col.attr('style', old_style).removeAttr('data-old_style');
					}
			
				} else {
			
					var col_height = col.outerHeight();
					var row_height = row.outerHeight();
					var row_top = row.offset().top;
		
					// If column shorter then content and page scrolled below the content's top
					if (col_height < row_height && _window_scroll_top + _fixed_rows_height > row_top) {
						
						var col_init = {
							'position': 'undefined',
							'top': 'auto',
							'bottom' : 'auto'
						};
						
						if (typeof TRX_ADDONS_STORAGE['scroll_offset_last'] == 'undefined') {
							TRX_ADDONS_STORAGE['col_top_last'] = row_top;
							TRX_ADDONS_STORAGE['scroll_offset_last'] = _window_scroll_top;
							TRX_ADDONS_STORAGE['scroll_dir_last'] = 1;
						}
						var scroll_dir = _window_scroll_top - TRX_ADDONS_STORAGE['scroll_offset_last'];
						scroll_dir = scroll_dir == 0
										? TRX_ADDONS_STORAGE['scroll_dir_last']
										: ( scroll_dir > 0 ? 1 : -1 );

						var col_big = col_height + 30 >= _window_height - _fixed_rows_height,
							col_top = col.offset().top;

						if (col_top < 0) {
							col_top = TRX_ADDONS_STORAGE['col_top_last'];
						}


						// If column height greater then window height
						if (col_big) {
		
							// If change scrolling dir
							if (scroll_dir != TRX_ADDONS_STORAGE['scroll_dir_last'] && col.css('position') == 'fixed') {
								col_init.top = col_top - row_top;
								col_init.position = 'absolute';
		
							// If scrolling down
							} else if (scroll_dir > 0) {
								if (_window_scroll_top + _window_height >= row_top + row_height + 30) {
									col_init.bottom = 0;
									col_init.position = 'absolute';
								} else if (_window_scroll_top + _window_height >= (col.css('position') == 'absolute' ? col_top : row_top) + col_height + 30) {
									col_init.bottom = 30;
									col_init.position = 'fixed';
								}
						
							// If scrolling up
							} else {
								if (_window_scroll_top + _fixed_rows_height <= col_top) {
									col_init.top = _fixed_rows_height;
									col_init.position = 'fixed';
								}
							}
						
						// If column height less then window height
						} else {
							if (_window_scroll_top + _fixed_rows_height >= row_top + row_height - col_height) {
								col_init.bottom = 0;
								col_init.position = 'absolute';
							} else {
								col_init.top = _fixed_rows_height;
								col_init.position = 'fixed';
							}
						}

						if (force && col_init.position == 'undefined' && col.css('position') == 'absolute') {
							col_init.position = 'absolute';
							if (col.css('top') != 'auto') {
								col_init.top = col.css('top');
							} else {
								col_init.bottom = col.css('bottom');
							}
						}

						if (col_init.position != 'undefined') {
							// Insert placeholder before this column
							var style = col.attr('style');
							if ( ! style ) style = '';
							if ( ! col.prev().hasClass('sc_column_fixed_placeholder') ) {
								col.css(col_init);
								TRX_ADDONS_STORAGE['scroll_dir_last'] = 0;
								col.before('<div class="sc_column_fixed_placeholder '+col.attr('class').replace('sc_column_fixed', '')+'"'
											+ (col.data('col') ? ' data-col="' + col.data('col') + '"' : '')
											+ '></div>');
							}
							// Detect horizontal position
							col_init.left = col_init.position == 'fixed' ? col.prev().offset().left : col.prev().position().left;
							col_init.width = col.prev().width() + parseFloat(col.prev().css('paddingLeft')) + parseFloat(col.prev().css('paddingRight'));
							// Set position
							if ( force 
								|| col.css('position') != col_init.position 
								|| TRX_ADDONS_STORAGE['scroll_dir_last'] != scroll_dir
								|| col.width() != col_init.width
							) {
								if (col.data('old_style') === undefined) {
									col.attr('data-old_style', style);
								}
								col.css(col_init);
							}
						}

						TRX_ADDONS_STORAGE['col_top_last'] = col_top;
						TRX_ADDONS_STORAGE['scroll_offset_last'] = _window_scroll_top;
						TRX_ADDONS_STORAGE['scroll_dir_last'] = scroll_dir;
		
					} else {
		
						// Unfix when page scrolling to top
						var old_style = col.data('old_style');
						if (old_style !== undefined) {
							col.attr('style', old_style).removeAttr('data-old_style');
							if ( col.prev().hasClass('sc_column_fixed_placeholder') ) {
								col.prev().remove();
							}
						}
					}
				}
			});
		}
	}

	// Stack sections
	$document.on('action.resize_trx_addons', trx_addons_stack_section);
	$document.on('action.scroll_trx_addons', trx_addons_stack_section);
	
	function trx_addons_stack_section(e, cont) {

		if ( $stack_sections.length == 0 ) return;

		var force = e.namespace == 'resize_trx_addons';
		var wso = _window_scroll_top + _fixed_rows_height;

		$stack_sections.each(function() {

			var row = jQuery(this);
			
			// Unfix on mobile layout (all columns are fullwidth)
			if ( _window_width < TRX_ADDONS_STORAGE['mobile_breakpoint_stacksections_off'] ) {

				if ( row.hasClass('sc_stack_section_fixed') ) {
					row.removeClass('sc_stack_section_fixed').css( { top: row.data('old-top') } ).prev().remove();
				}
		
			} else {
		
				// If section is fixed
				if ( row.hasClass('sc_stack_section_fixed') ) {
					var row_holder = row.prev(),
						row_height = row_holder.outerHeight(),
						row_top    = row_holder.offset().top;

					// Unfix row
					if ( row_top > wso ) {
						row.removeClass('sc_stack_section_fixed').css( { top: row.data('old-top') } );
						if ( row.hasClass( 'sc_stack_section_effect_fade' ) ) {
							row.css( { 'opacity': 0 } );
						}
						row_holder.remove();

					// Set holder height
					} else {
						if (force) {
							var row_height = row.outerHeight();
							row_holder.height(row_height);
							row.css( { top: _fixed_rows_height + 'px !important' } );
						}
						if ( row.hasClass( 'sc_stack_section_effect_fade' ) ) {
							if ( wso - row_top <= row_height ) {
								row.css( { 'opacity': Math.max(0, Math.min( 1, ( wso - row_top ) / row_height ) ) } );
							} else {
								row.css( { 'opacity': 1 } );
							}
						}
					}

				// If section is not fixed
				} else {
					var row_top = row.offset().top;
					if ( row_top <= wso ) {
						var row_height = row.outerHeight();
						row
							.addClass('sc_stack_section_fixed')
							.data( 'old-top', row.css('top') )
							.css( { top: _fixed_rows_height+'px'} )
							.before('<div class="sc_stack_section_placeholder" style="height:'+row_height+'px;"></div>');
					}
				}
			}

		} );
	}


	// Resize actions
	//==============================================

	// Do actions when window is resized
	window.trx_addons_resize_actions = function(cont) {

		// Update global values
		_window_height     = $window.height();
		_window_width      = $window.width();
		_window_scroll_top = $window.scrollTop();
		_document_height   = $document.height();
		_adminbar_height   = trx_addons_adminbar_height();
		
		update_fixed_rows_height();

		if (cont===undefined) cont = $body;

		// Call theme/plugins specific action (if exists)
		$document.trigger('action.before_resize_trx_addons', [cont] );
		$document.trigger('action.resize_trx_addons', [cont] );
		$document.trigger('action.after_resize_trx_addons', [cont] );
	};

	// Fit video frames to document width
	$document.on('action.resize_trx_addons', trx_addons_resize_video);
	function trx_addons_resize_video(e, cont) {
		if ( $video_tags.length > 0 ) {
			$video_tags.each(function() {
				var $self = jQuery(this);
				// If item now invisible
				if ($self.parents('div:hidden,section:hidden,article:hidden').length > 0 || $self.hasClass('trx_addons_noresize')) {
					return;
				}
				var video = $self.addClass('trx_addons_resize').eq(0);
				var ratio = (video.data('ratio') !== undefined ? video.data('ratio').split(':') : [16,9]);
				ratio = ratio.length!=2 || ratio[0]==0 || ratio[1]==0 ? 16/9 : ratio[0]/ratio[1];
				var mejs_cont = video.parents('.mejs-video').eq(0);
				var w_attr = video.data('width');
				var h_attr = video.data('height');
				if (!w_attr || !h_attr) {
					w_attr = video.attr('width');
					h_attr = video.attr('height');
					if ((!w_attr || !h_attr) && mejs_cont.length > 0) {
						w_attr = mejs_cont.width();
						h_attr = mejs_cont.height();
					}
					if (!w_attr || !h_attr) return;
					video.data({'width': w_attr, 'height': h_attr});
				}
				var percent = (''+w_attr).substr(-1)=='%';
				w_attr = parseInt(w_attr, 10);
				h_attr = parseInt(h_attr, 10);
				var w_real = Math.round(mejs_cont.length > 0 
										? Math.min(percent ? 10000 : w_attr, mejs_cont.parents('div,article').eq(0).width()) 
										: Math.min(percent ? 10000 : w_attr, video.parents('div,article').eq(0).width()) 
									   ),
					h_real = Math.round(percent ? w_real/ratio : w_real/w_attr*h_attr);
				if (parseInt(video.attr('data-last-width'), 10)==w_real)
					return;
				if (percent) {
					video.height(h_real);
				} else if (video.parents('.wp-video-playlist').length > 0) {
					if (mejs_cont.length === 0) {
						video.attr({'width': w_real, 'height': h_real});
					}
				} else {
					video.attr({'width': w_real, 'height': h_real}).css({'width': w_real+'px', 'height': h_real+'px'});
					if (mejs_cont.length > 0) {
						trx_addons_set_mejs_player_dimensions(video, w_real, h_real);
					}
				}
				video.attr('data-last-width', w_real);
			});
		}
		if ( $iframe_tags.length > 0 ) {
			$iframe_tags.each(function() {
				var $self = jQuery(this);
				// If item now invisible
				if ($self.addClass('trx_addons_resize').parents('div:hidden,section:hidden,article:hidden').length > 0 || $self.hasClass('trx_addons_noresize')) {
					return;
				}
				var iframe = $self.eq(0),
					iframe_src = iframe.attr('src') ? iframe.attr('src') : iframe.data('src');
				if (iframe_src === undefined || iframe_src.indexOf('soundcloud') > 0) return;
				var ratio = (iframe.data('ratio') !== undefined 
								? iframe.data('ratio').split(':') 
								: (iframe.parent().data('ratio') !== undefined 
									? iframe.parent().data('ratio').split(':') 
									: (iframe.find('[data-ratio]').length>0 
										? iframe.find('[data-ratio]').data('ratio').split(':') 
										: [16,9]
										)
									)
								);
				ratio = ratio.length!=2 || ratio[0]==0 || ratio[1]==0 ? 16/9 : ratio[0]/ratio[1];
				var w_attr = iframe.attr('width');
				var h_attr = iframe.attr('height');
				if (!w_attr || !h_attr) {
					return;
				}
				var percent = (''+w_attr).substr(-1)=='%';
				w_attr = parseInt(w_attr, 10);
				h_attr = parseInt(h_attr, 10);
				var par    = iframe.parents('div,section').eq(0),
					pw     = par.width(),
					ph     = par.height(),
					w_real = pw,
					h_real = Math.round(percent ? w_real/ratio : w_real/w_attr*h_attr);
				if (par.css('position') == 'absolute' && h_real > ph) {
					h_real = ph;
					w_real = Math.round(percent ? h_real*ratio : h_real*w_attr/h_attr)
				}
				if (parseInt(iframe.attr('data-last-width'), 10)==w_real) return;
				iframe.css({'width': w_real+'px', 'height': h_real+'px'});
				iframe.attr('data-last-width', w_real);
			});
		}
	}
	
	
	// Set Media Elements player dimensions
	function trx_addons_set_mejs_player_dimensions(video, w, h) {
		if (mejs) {
			for (var pl in mejs.players) {
				if (mejs.players[pl].media.src == video.attr('src')) {
					if (mejs.players[pl].media.setVideoSize) {
						mejs.players[pl].media.setVideoSize(w, h);
					} else if (mejs.players[pl].media.setSize) {
						mejs.players[pl].media.setSize(w, h);
					}
					mejs.players[pl].setPlayerSize(w, h);
					mejs.players[pl].setControlsSize();
				}
			}
		}
	}

} );


/* Global functions */
(function() {
	"use strict";

	/* Paint arc on canvas with digits (used in the shortcodes and reviews) */
	window.trx_addons_draw_arc_on_canvas = function(item, value) {
	
		var canvas = item.find('canvas');
		if (canvas.length == 0) return;
		
		var digits = canvas.next();
		var brd = parseInt(digits.css('border-top-width'), 10);
		var w = Math.ceil(digits.width()+2*brd);
	
		var needRepaint = false;
		if (canvas.attr('width') != w) {
			needRepaint = true;
			canvas.attr({
				'width': w,
				'height': w
			});
		}
	
		if (item.data('old-value') == value && !needRepaint) return;
		item.data('old-value', value);
		
		var percent = value * 100 / canvas.data('max-value');
		var angle = 360 * percent / 100;
		var Ar = angle * Math.PI / 180;
	
		var canvas_dom = canvas.get(0);
		var context = canvas_dom.getContext('2d');
		var r = (w - brd) / 2;
		var cx = w / 2;
		var cy = w / 2;
	
		context.beginPath();
		context.clearRect(0, 0, w, w);
		context.arc(cx, cy, r, 0, Ar, false);
		context.imageSmoothingEnabled= true;
		context.lineWidth = brd;
		context.strokeStyle = canvas.data('color');
		context.stroke();
	};

})();